@@ -35,7 +35,7 @@ except Exception as e:
     LastFM = None
 
 from lollypop.utils import is_gnome, is_unity
-from lollypop.define import ArtSize
+from lollypop.define import ArtSize, Type
 from lollypop.window import Window
 from lollypop.database import Database
 from lollypop.player import Player
@@ -52,7 +52,6 @@ from lollypop.playlists import Playlists
 from lollypop.radios import Radios
 from lollypop.collectionscanner import CollectionScanner
 from lollypop.fullscreen import FullScreen
-from lollypop.mpd import MpdServerDaemon
 
 
 class Application(Gtk.Application):
@@ -71,10 +70,11 @@ class Application(Gtk.Application):
                             self,
                             application_id='org.gnome.Lollypop',
                             flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
+        os.environ['PULSE_PROP_media.role'] = 'music'
+        os.environ['PULSE_PROP_application.icon_name'] = 'lollypop'
         self.cursors = {}
         self.window = None
         self.notify = None
-        self.mpd = None
         self.lastfm = None
         self.debug = False
         self._externals_count = 0
@@ -136,10 +136,6 @@ class Application(Gtk.Application):
         self.art = Art()
         if not self.settings.get_value('disable-mpris'):
             MPRIS(self)
-        if not self.settings.get_value('disable-mpd'):
-            self.mpd = MpdServerDaemon(
-                               self.settings.get_value('mpd-eth').get_string(),
-                               self.settings.get_value('mpd-port').get_int32())
         if not self.settings.get_value('disable-notifications'):
             self.notify = NotificationManager()
 
@@ -177,7 +173,7 @@ class Application(Gtk.Application):
             if is_gnome() or is_unity():
                 self.set_app_menu(menu)
             self.window = Window(self)
-            # If not GNOME add menu to toolbar
+            # If not GNOME/Unity add menu to toolbar
             if not is_gnome() and not is_unity():
                 self.window.setup_menu(menu)
             self.window.connect('delete-event', self._hide_on_delete)
@@ -193,12 +189,26 @@ class Application(Gtk.Application):
             return
         if self.settings.get_value('save-state'):
             self.window.save_view_state()
+            # Save current track
             if self.player.current_track.id is None:
                 track_id = -1
+            elif self.player.current_track.id == Type.RADIOS:
+                radios = Radios()
+                track_id = radios.get_id(
+                                        self.player.current_track.album_artist)
             else:
                 track_id = self.player.current_track.id
             self.settings.set_value('track-id', GLib.Variant('i',
                                                              track_id))
+            # Save current playlist
+            if self.player.current_track.id == Type.RADIOS:
+                playlist_ids = [Type.RADIOS]
+            elif not self.player.get_user_playlist_ids():
+                playlist_ids = []
+            else:
+                playlist_ids = self.player.get_user_playlist_ids()
+            self.settings.set_value('playlist-ids', GLib.Variant('ai',
+                                                                 playlist_ids))
         self.player.stop_all()
         if self.window:
             self.window.stop_all()
@@ -208,8 +218,6 @@ class Application(Gtk.Application):
         """
             Quit lollypop
         """
-        if self.mpd is not None:
-            self.mpd.quit()
         if self.scanner.is_locked():
             self.scanner.stop()
             GLib.idle_add(self.quit)
@@ -295,7 +303,8 @@ class Application(Gtk.Application):
         """
         self.player.load_external(uri)
         if self._externals_count == 0:
-            self.player.set_party(False)
+            if self.player.is_party():
+                self.player.set_party(False)
             self.player.play_first_external()
         self._externals_count += 1
 
