@@ -21,6 +21,7 @@ from lollypop.inotify import Inotify
 from lollypop.define import Lp
 from lollypop.sqlcursor import SqlCursor
 from lollypop.tagreader import ScannerTagReader
+from lollypop.database_history import History
 from lollypop.utils import is_audio, is_pls, debug
 
 
@@ -30,9 +31,9 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
     """
     __gsignals__ = {
         'scan-finished': (GObject.SignalFlags.RUN_FIRST, None, ()),
-        'artist-update': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
-        'genre-update': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        'album-modified': (GObject.SignalFlags.RUN_FIRST, None, (int,))
+        'artist-added': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
+        'genre-added': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
+        'album-update': (GObject.SignalFlags.RUN_FIRST, None, (int,))
     }
 
     def __init__(self):
@@ -44,6 +45,7 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
 
         self._thread = None
         self._inotify = None
+        self._history = None
         if Lp().settings.get_value('auto-update'):
             self._inotify = Inotify()
         self._progress = None
@@ -131,7 +133,6 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
         """
             Notify from main thread when scan finished
         """
-        Lp().settings.set_value('db-mtime', GLib.Variant('i', int(time())))
         self.stop()
         self.emit("scan-finished")
         if self._missing_codecs is not None:
@@ -145,13 +146,16 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
             @param paths as [string], paths to scan
             @thread safe
         """
-        self._new_albums = []
+        if self._history is None:
+            self._history = History()
         mtimes = Lp().tracks.get_mtimes()
         orig_tracks = Lp().tracks.get_paths()
-        is_empty = len(orig_tracks) == 0
+        was_empty = len(orig_tracks) == 0
+
+        (new_tracks, new_dirs, count) = self._get_objects_for_paths(paths)
+        count += len(orig_tracks)
 
         # Add monitors on dirs
-        (new_tracks, new_dirs, count) = self._get_objects_for_paths(paths)
         if self._inotify is not None:
             for d in new_dirs:
                 self._inotify.add_monitor(d)
@@ -163,59 +167,50 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
                     return
                 GLib.idle_add(self._update_progress, i, count)
                 try:
-                    mtime = int(os.path.getmtime(filepath))
-                    if filepath not in orig_tracks:
-                        try:
-                            debug("Adding file: %s" % filepath)
-                            infos = self.get_infos(filepath)
-                            self._add2db(filepath, mtime, infos)
-                        except Exception as e:
-                            debug("Error scanning: %s, %s" % (filepath, e))
-                            string = "%s" % e
-                            if string.startswith('gst-core-error-quark'):
-                                self._missing_codecs = filepath
-                    else:
-                        # Update tags by removing song and readd it
-                        if mtime != mtimes[filepath]:
-                            debug("Adding file: %s" % filepath)
-                            infos = self.get_infos(filepath)
-                            track_id = Lp().tracks.get_id_by_path(filepath)
-                            self._del_from_db(track_id)
-                            if infos is not None:
-                                self._add2db(filepath, mtime, infos)
-                            else:
-                                print("Can't get infos for ", filepath)
+                    debug("Adding file: %s" % filepath)
+                    # If songs exists and mtime unchanged, continue,
+                    # else rescan
+                    if filepath in orig_tracks:
                         orig_tracks.remove(filepath)
-
+                        i += 1
+                        mtime = int(os.path.getmtime(filepath))
+                        if mtime <= mtimes[filepath]:
+                            i += 1
+                            continue
+                        else:
+                            self._del_from_db(filepath)
+                    infos = self.get_infos(filepath)
+                    # On first scan, use modification time
+                    # Else, use current time
+                    if was_empty:
+                        mtime = int(os.path.getmtime(filepath))
+                    else:
+                        mtime = int(time())
+                    self._add2db(filepath, infos, mtime)
                 except Exception as e:
-                    print(ascii(filepath))
-                    print("CollectionScanner::_scan(): %s" % e)
+                    debug("Error scanning: %s, %s" % (filepath, e))
+                    string = "%s" % e
+                    if string.startswith('gst-core-error-quark'):
+                        self._missing_codecs = filepath
                 i += 1
 
-            # Restore stats for new albums
-            if not is_empty:
-                for album_id in self._new_albums:
-                    duration = Lp().albums.get_duration(album_id, None)
-                    count = Lp().albums.get_count(album_id, None)
-                    value = Lp().albums.get_stats(duration, count)
-                    if value is not None:
-                        Lp().albums.set_popularity(album_id, value[0])
-                        Lp().albums.set_mtime(album_id, value[1])
-
             # Clean deleted files
             for filepath in orig_tracks:
-                track_id = Lp().tracks.get_id_by_path(filepath)
-                self._del_from_db(track_id)
+                i += 1
+                GLib.idle_add(self._update_progress, i, count)
+                self._del_from_db(filepath)
 
             sql.commit()
         GLib.idle_add(self._finish)
+        del self._history
+        self._history = None
 
-    def _add2db(self, filepath, mtime, infos):
+    def _add2db(self, filepath, infos, mtime):
         """
             Add new file to db with informations
             @param filepath as string
-            @param file modification time as int
             @param infos as GstPbutils.DiscovererInfo
+            @param mtime as int
             @return track id as int
         """
         tags = infos.get_tags()
@@ -230,7 +225,18 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
         tracknumber = self.get_tracknumber(tags)
         year = self.get_year(tags)
         duration = int(infos.get_duration()/1000000000)
+        name = GLib.path_get_basename(filepath)
 
+        # If no artists tag, use album artist
+        if artists == '':
+            artists = album_artist
+
+        # Restore stats
+        (track_pop, track_ltime, amtime, album_pop) = self._history.get(
+                                                            name, duration)
+        # If nothing in stats, set mtime
+        if amtime == 0:
+            amtime = mtime
         (artist_ids, new_artist_ids) = self.add_artists(artists,
                                                         album_artist,
                                                         sortname)
@@ -244,26 +250,17 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
             album_artist_id = artist_ids[0]
             no_album_artist = True
 
-        (album_id, new) = self.add_album(album_name, album_artist_id,
-                                         no_album_artist, year, filepath, 0,
-                                         mtime)
-        if new:
-            self._new_albums.append(album_id)
+        (album_id, new_album) = self.add_album(album_name, album_artist_id,
+                                               no_album_artist, year, filepath,
+                                               album_pop, amtime)
 
         (genre_ids, new_genre_ids) = self.add_genres(genres, album_id)
 
-        # Restore stats
-        value = Lp().tracks.get_stats(filepath, duration)
-        if value is None:
-            popularity = 0
-            ltime = 0
-        else:
-            popularity = value[0]
-            ltime = value[1]
         # Add track to db
         track_id = Lp().tracks.add(title, filepath, duration,
                                    tracknumber, discnumber,
-                                   album_id, year, popularity, ltime, mtime)
+                                   album_id, year, track_pop,
+                                   track_ltime, mtime)
         self.update_track(track_id, artist_ids, genre_ids)
 
         # Notify about new artists/genres
@@ -271,25 +268,36 @@ class CollectionScanner(GObject.GObject, ScannerTagReader):
             with SqlCursor(Lp().db) as sql:
                 sql.commit()
             for genre_id in new_genre_ids:
-                GLib.idle_add(self.emit, 'genre-update', genre_id)
+                GLib.idle_add(self.emit, 'genre-added', genre_id)
             for artist_id in new_artist_ids:
-                GLib.idle_add(self.emit, 'artist-update', artist_id, album_id)
+                GLib.idle_add(self.emit, 'artist-added', artist_id, album_id)
         return track_id
 
-    def _del_from_db(self, track_id):
+    def _del_from_db(self, filepath):
         """
             Delete track from db
-            @param track_id as int
+            @param filepath as str
         """
+        name = GLib.path_get_basename(filepath)
+        track_id = Lp().tracks.get_id_by_path(filepath)
         album_id = Lp().tracks.get_album_id(track_id)
         genre_ids = Lp().tracks.get_genre_ids(track_id)
         album_artist_id = Lp().albums.get_artist_id(album_id)
         artist_ids = Lp().tracks.get_artist_ids(track_id)
+        popularity = Lp().tracks.get_popularity(track_id)
+        ltime = Lp().tracks.get_ltime(track_id)
+        mtime = Lp().albums.get_mtime(album_id)
+        duration = Lp().tracks.get_duration(track_id)
+        album_popularity = Lp().albums.get_popularity(album_id)
+        self._history.add(name, duration, popularity,
+                          ltime, mtime, album_popularity)
         Lp().tracks.remove(track_id)
         Lp().tracks.clean(track_id)
         modified = Lp().albums.clean(album_id)
         if modified:
-            GLib.idle_add(self.emit, 'album-modified', album_id)
+            with SqlCursor(Lp().db) as sql:
+                sql.commit()
+            GLib.idle_add(self.emit, 'album-update', album_id)
         for artist_id in [album_artist_id] + artist_ids:
             Lp().artists.clean(artist_id)
         for genre_id in genre_ids:
