@@ -12,16 +12,16 @@
 
 from gi.repository import Gtk, Gio, GLib
 
-from threading import Thread, Lock
 from gettext import gettext as _
 
-from lollypop.define import Lp, Type
+from lollypop.define import Lp, Type, SelectionMode
+from lollypop.loader import Loader
 from lollypop.selectionlist import SelectionList
 from lollypop.view_container import ViewContainer
 from lollypop.view_albums import AlbumsView
 from lollypop.view_artist import ArtistView
 from lollypop.view_radios import RadiosView
-from lollypop.view_playlists import PlaylistView
+from lollypop.view_playlists import PlaylistsView
 from lollypop.view_playlists import PlaylistsManageView, PlaylistEditView
 from lollypop.view_device import DeviceView, DeviceLocked
 
@@ -33,47 +33,6 @@ class Device:
     uri = None
 
 
-class Loader(Thread):
-    """
-        Helper to load data on a separate thread and
-        dispatch it to the UI thread
-    """
-    active = {}
-    active_lock = Lock()
-
-    def __init__(self, target, view=None, on_finished=None):
-        Thread.__init__(self)
-        self.daemon = True
-        self._target = target
-        self._view = view
-        self._on_finished = on_finished
-        self._invalidated = False
-        self._invalidated_lock = Lock()
-
-    def is_invalidated(self):
-        with self._invalidated_lock:
-            return self._invalidated
-
-    def invalidate(self):
-        with self._invalidated_lock:
-            self._invalidated = True
-
-    def run(self):
-        with Loader.active_lock:
-            active = Loader.active.get(self._view, None)
-            if active:
-                active.invalidate()
-            Loader.active[self._view] = self
-        result = self._target()
-        if not self.is_invalidated():
-            if self._on_finished:
-                GLib.idle_add(self._on_finished, (result))
-            elif self._view:
-                GLib.idle_add(self._view.populate, (result))
-            with Loader.active_lock:
-                Loader.active.pop(self._view, None)
-
-
 class Container:
     """
         Container for main view
@@ -178,6 +137,7 @@ class Container:
         view.show()
         self._stack.add(view)
         self._stack.set_visible_child(view)
+        self._stack.clean_old_views(view)
         view.populate()
 
     def main_widget(self):
@@ -222,6 +182,14 @@ class Container:
                 self._stack.clean_old_views(child)
                 break
 
+    def update_overlays(self):
+        """
+            Update current view
+        """
+        view = self._stack.get_visible_child()
+        if view:
+            view.update_overlays()
+
     def update_view(self):
         """
             Update current view
@@ -309,9 +277,9 @@ class Container:
         vgrid = Gtk.Grid()
         vgrid.set_orientation(Gtk.Orientation.VERTICAL)
 
-        self._list_one = SelectionList()
+        self._list_one = SelectionList(SelectionMode.LIMITED)
         self._list_one.show()
-        self._list_two = SelectionList()
+        self._list_two = SelectionList(SelectionMode.NORMAL)
         self._list_one.connect('item-selected', self._on_list_one_selected)
         self._list_one.connect('populated', self._on_list_populated)
         self._list_two.connect('item-selected', self._on_list_two_selected)
@@ -387,8 +355,8 @@ class Container:
             @return True if hard scan is running
         """
         Lp().scanner.connect('scan-finished', self.on_scan_finished)
-        Lp().scanner.connect('genre-update', self._add_genre)
-        Lp().scanner.connect('artist-update', self._add_artist)
+        Lp().scanner.connect('genre-added', self._add_genre)
+        Lp().scanner.connect('artist-added', self._add_artist)
 
     def _update_playlists(self, playlists, playlist_id):
         """
@@ -523,7 +491,6 @@ class Container:
         playlists.append((Type.RECENTS, _("Recently played")))
         playlists.append((Type.NEVER, _("Never played")))
         playlists.append((Type.RANDOMS, _("Random tracks")))
-        playlists.append((Type.MPD, _("Network control")))
         playlists.append((Type.SEPARATOR, ''))
         playlists += Lp().playlists.get()
         if update:
@@ -559,12 +526,13 @@ class Container:
             @param genre id as int
         """
         def load():
-            if artist_ids and artist_ids[0] == Type.COMPILATIONS:
-                albums = Lp().albums.get_compilations(genre_ids)
-            elif genre_ids and genre_ids[0] == Type.ALL:
+            if genre_ids and genre_ids[0] == Type.ALL:
                 albums = Lp().albums.get_ids(artist_ids, [])
             else:
-                albums = Lp().albums.get_ids(artist_ids, genre_ids)
+                albums = []
+                if artist_ids and artist_ids[0] == Type.COMPILATIONS:
+                    albums += Lp().albums.get_compilations(genre_ids)
+                albums += Lp().albums.get_ids(artist_ids, genre_ids)
             return albums
 
         view = ArtistView(artist_ids, genre_ids)
@@ -604,7 +572,10 @@ class Container:
                 albums += Lp().albums.get_ids([], genre_ids)
             return albums
 
-        view = AlbumsView(genre_ids, is_compilation)
+        artist_ids = []
+        if is_compilation:
+            artist_ids.append(Type.COMPILATIONS)
+        view = AlbumsView(genre_ids, artist_ids)
         loader = Loader(target=load, view=view)
         loader.start()
         view.show()
@@ -612,42 +583,41 @@ class Container:
         self._stack.set_visible_child(view)
         self._stack.clean_old_views(view)
 
-    def _update_view_playlists(self, playlist_id):
+    def _update_view_playlists(self, playlist_ids=[]):
         """
             Update current view with playlist view
-            @param playlist id as int
+            @param playlist ids as [int]
         """
         def load():
-            if playlist_id == Lp().player.get_user_playlist_id():
-                tracks = [t.id for t in Lp().player.get_user_playlist()]
-            elif playlist_id == Type.POPULARS:
-                tracks = Lp().tracks.get_populars()
-            elif playlist_id == Type.RECENTS:
-                tracks = Lp().tracks.get_recently_listened_to()
-            elif playlist_id == Type.NEVER:
-                tracks = Lp().tracks.get_never_listened_to()
-            elif playlist_id == Type.RANDOMS:
-                tracks = Lp().tracks.get_randoms()
-            else:
-                tracks = Lp().playlists.get_tracks_ids(playlist_id)
-            return tracks
+            track_ids = []
+            for playlist_id in playlist_ids:
+                if playlist_id == Type.POPULARS:
+                    tracks = Lp().tracks.get_populars()
+                elif playlist_id == Type.RECENTS:
+                    tracks = Lp().tracks.get_recently_listened_to()
+                elif playlist_id == Type.NEVER:
+                    tracks = Lp().tracks.get_never_listened_to()
+                elif playlist_id == Type.RANDOMS:
+                    tracks = Lp().tracks.get_randoms()
+                else:
+                    tracks = Lp().playlists.get_tracks_ids(playlist_id)
+                for track_id in tracks:
+                    if track_id not in track_ids:
+                        track_ids.append(track_id)
+            return track_ids
 
         view = None
-        if playlist_id is not None:
-            view = PlaylistView(playlist_id)
+        if playlist_ids:
+            view = PlaylistsView(playlist_ids)
+            loader = Loader(target=load, view=view)
+            loader.start()
         else:
             view = PlaylistsManageView(Type.NONE, None, False)
-        if view:
-            # Management or user playlist
-            if playlist_id is None:
-                view.populate()
-            else:
-                loader = Loader(target=load, view=view)
-                loader.start()
-            view.show()
-            self._stack.add(view)
-            self._stack.set_visible_child(view)
-            self._stack.clean_old_views(view)
+            view.populate()
+        view.show()
+        self._stack.add(view)
+        self._stack.set_visible_child(view)
+        self._stack.clean_old_views(view)
 
     def _update_view_radios(self):
         """
@@ -711,10 +681,9 @@ class Container:
             return
         if selected_ids[0] == Type.PLAYLISTS:
             self._list_two.clear()
-            self._list_two.set_mode(Gtk.SelectionMode.SINGLE)
             self._list_two.show()
             if not self._list_two.will_be_selected():
-                self._update_view_playlists(None)
+                self._update_view_playlists()
             self._setup_list_playlists(False)
         elif Type.DEVICES - 999 < selected_ids[0] < Type.DEVICES:
             self._list_two.hide()
@@ -732,13 +701,13 @@ class Container:
             self._list_two.hide()
             if selected_ids[0] == Type.ALL:
                 self._update_view_albums(selected_ids)
-            elif selected_ids[0] == Type.COMPILATIONS:
+            elif selected_ids[0] == Type.COMPILATIONS and\
+                    len(selected_ids) == 1:
                 self._update_view_albums([], True)
             else:
                 self._update_view_artists(selected_ids, [])
         else:
             self._list_two.clear()
-            self._list_two.set_mode(Gtk.SelectionMode.MULTIPLE)
             self._setup_list_artists(self._list_two, selected_ids, False)
             self._list_two.show()
             if not self._list_two.will_be_selected():
@@ -762,8 +731,9 @@ class Container:
         if not selected_ids or not genre_ids:
             return
         if genre_ids[0] == Type.PLAYLISTS:
-            self._update_view_playlists(selected_ids[0])
-        elif selected_ids[0] == Type.COMPILATIONS:
+            self._update_view_playlists(selected_ids)
+        elif selected_ids[0] == Type.COMPILATIONS and\
+                len(selected_ids) == 1:
             self._update_view_albums(genre_ids, True)
         else:
             self._update_view_artists(selected_ids, genre_ids)
