@@ -96,17 +96,6 @@ class AlbumsDatabase:
             sql.execute("UPDATE albums SET path=? WHERE rowid=?",
                         (path, album_id))
 
-    def set_mtime(self, album_id, mtime):
-        """
-            Set mtime
-            @param album_id as int
-            @param mtime as int
-            @warning: commit needed
-        """
-        with SqlCursor(Lp().db) as sql:
-            sql.execute("UPDATE albums set mtime=? WHERE rowid=?",
-                        (mtime, album_id))
-
     def set_popularity(self, album_id, popularity, commit=False):
         """
             Set popularity
@@ -123,6 +112,21 @@ class AlbumsDatabase:
             except:  # Database is locked
                 pass
 
+    def get_mtime(self, album_id):
+        """
+            Get modification time
+            @param album_id as int
+            @return modification time as int
+        """
+        with SqlCursor(Lp().db) as sql:
+            result = sql.execute("SELECT mtime FROM albums WHERE\
+                                 rowid=?", (album_id,))
+
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
+
     def get_popularity(self, album_id):
         """
             Get popularity
@@ -439,37 +443,6 @@ class AlbumsDatabase:
                 return v[0]
             return 0
 
-    def get_count_for_disc(self, album_id, genre_ids, disc):
-        """
-            Get number of tracks for album_id/disc
-            @param album id as int
-            @param genre ids as [int]
-            @param disc number as int
-            @return list of int
-        """
-        with SqlCursor(Lp().db) as sql:
-            if genre_ids and genre_ids[0] > 0:
-                filters = (album_id, disc)
-                filters += tuple(genre_ids)
-                request = "SELECT COUNT(1)\
-                           FROM tracks, track_genres\
-                           WHERE tracks.album_id=?\
-                           AND track_genres.track_id = tracks.rowid\
-                           AND discnumber=? AND ("
-                for genre_id in genre_ids:
-                    request += "track_genres.genre_id=? OR "
-                request += "1=0)"
-                result = sql.execute(request, filters)
-            else:
-                result = sql.execute("SELECT COUNT(1)\
-                                      FROM tracks\
-                                      WHERE tracks.album_id=?\
-                                      AND discnumber=?", (album_id, disc))
-            v = result.fetchone()
-            if v is not None:
-                return v[0]
-            return 0
-
     def get_discs(self, album_id, genre_ids):
         """
             Get disc numbers
@@ -595,7 +568,8 @@ class AlbumsDatabase:
             # Get albums for all artists
             if not artist_ids and not genre_ids:
                 result = sql.execute(
-                                 "SELECT albums.rowid FROM albums, artists\
+                                 "SELECT DISTINCT albums.rowid\
+                                  FROM albums, artists\
                                   WHERE artists.rowid=albums.artist_id\
                                   ORDER BY artists.sortname COLLATE NOCASE,\
                                   albums.year,\
@@ -603,7 +577,7 @@ class AlbumsDatabase:
             # Get albums for genre
             elif not artist_ids:
                 genres = tuple(genre_ids)
-                request = "SELECT albums.rowid FROM albums,\
+                request = "SELECT DISTINCT albums.rowid FROM albums,\
                            album_genres, artists\
                            WHERE artists.rowid=artist_id\
                            AND album_genres.album_id=albums.rowid AND ("
@@ -666,7 +640,8 @@ class AlbumsDatabase:
             else:
                 filters = (Type.COMPILATIONS,)
                 filters += tuple(genre_ids)
-                request = "SELECT albums.rowid FROM albums, album_genres\
+                request = "SELECT DISTINCT albums.rowid\
+                           FROM albums, album_genres\
                            WHERE album_genres.album_id=albums.rowid\
                            AND albums.artist_id=? AND ( "
                 for genre_id in genre_ids:
@@ -744,40 +719,6 @@ class AlbumsDatabase:
                 return v[0]
             return 0
 
-    def get_stats(self, duration, count):
-        """
-            Get stats for album with same duration and track count
-            @param path as str
-            @param duration as int
-            @return (popularity, mtime) as (int, int)
-        """
-        with SqlCursor(Lp().db) as sql:
-            try:
-                sql.execute("CREATE TEMP TABLE stats (album_id INT,\
-                                                      count INT,\
-                                                      duration INT)")
-            except:
-                sql.execute("DELETE FROM stats")
-            sql.execute("INSERT INTO stats (album_id, count, duration)\
-                            SELECT album_id,\
-                                   COUNT(1),\
-                                   SUM(tracks.duration)\
-                            FROM tracks GROUP BY album_id")
-            result = sql.execute("SELECT album_id\
-                                  FROM stats\
-                                  WHERE count=? and duration=?",
-                                 (count, duration))
-            v = result.fetchone()
-            ret = None
-            if v is not None:
-                result = sql.execute("SELECT popularity, mtime\
-                                      FROM albums\
-                                      WHERE rowid=?", (v[0],))
-                v = result.fetchone()
-                if v is not None:
-                    ret = v
-            return ret
-
     def clean(self, album_id):
         """
             Clean database for album id
