@@ -10,7 +10,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib
 
 from gettext import gettext as _
 import itertools
@@ -251,6 +250,12 @@ class TracksDatabase:
                                   AND track_artists.artist_id=artists.rowid",
                                  (track_id,))
             artists = [row[0] for row in result]
+            album_id = self.get_album_id(track_id)
+            artist_id = Lp().albums.get_artist_id(album_id)
+            if artist_id != Type.COMPILATIONS:
+                artist_name = Lp().albums.get_artist_name(album_id)
+                if artist_name not in artists:
+                    artists.insert(0, artist_name)
             return ", ".join(artists)
 
     def get_genre_ids(self, track_id):
@@ -529,7 +534,34 @@ class TracksDatabase:
         with SqlCursor(Lp().db) as sql:
             result = sql.execute("SELECT popularity FROM tracks WHERE\
                                  rowid=?", (track_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
 
+    def get_ltime(self, track_id):
+        """
+            Get listen time
+            @param track id  as int
+            @return listen time as int
+        """
+        with SqlCursor(Lp().db) as sql:
+            result = sql.execute("SELECT ltime FROM tracks WHERE\
+                                 rowid=?", (track_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
+
+    def get_mtime(self, track_id):
+        """
+            Get modification time
+            @param track id  as int
+            @return popularity as int
+        """
+        with SqlCursor(Lp().db) as sql:
+            result = sql.execute("SELECT mtime FROM tracks WHERE\
+                                 rowid=?", (track_id,))
             v = result.fetchone()
             if v is not None:
                 return v[0]
@@ -571,25 +603,6 @@ class TracksDatabase:
                                  ('%' + searched + '%',))
             return list(result)
 
-    def get_stats(self, path, duration):
-        """
-            Get stats for track with filename and duration
-            @param path as str
-            @param duration as int
-            @return (popularity, mtime) as (int, int)
-        """
-        with SqlCursor(Lp().db) as sql:
-            name = GLib.path_get_basename(path)
-            result = sql.execute("SELECT popularity, ltime\
-                                  FROM tracks\
-                                  WHERE filepath LIKE ?\
-                                  AND duration=?",
-                                 ('%' + name + '%', duration))
-            v = result.fetchone()
-            if v is not None:
-                return v
-            return None
-
     def search_track(self, artist, title):
         """
             Get track id for artist and title
