@@ -38,15 +38,20 @@ class NextContext:
     NONE = 0             # Continue playback
     STOP_TRACK = 1       # Stop after current track
     STOP_ALBUM = 2       # Stop after current album
-    STOP_ARTIST = 3      # Stop after current artist
+    STOP_ALL = 3         # Stop
     START_NEW_ALBUM = 4  # Start a new album
 
 
+class SelectionMode:
+    NORMAL = 0
+    LIMITED = 1
+
+
 # Represent playback context
 class PlayContext:
-    genre_ids = []
-    artist_ids = []
+    genre_ids = {}
     next_track = None  # Needed to return from queue
+    prev_track = None  # Needed to return from queue
     next = NextContext.NONE
 
 
@@ -85,8 +90,8 @@ class Type:
     PLAYLISTS = -5
     RADIOS = -6
     EXTERNALS = -7
-    ALL = -8
-    MPD = -996
+    SEARCH = -8
+    ALL = -9
     LOVED = -997
     NEVER = -998
     DEVICES = -1000
