@@ -11,9 +11,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-
 from gi.repository import GLib
 
+from cgi import escape
+
 from lollypop.radios import Radios
 from lollypop.define import Lp, Type
 
@@ -128,7 +129,7 @@ class Album(Base):
     FIELDS = ['name', 'artist_name', 'artist_id', 'year', 'path', 'duration']
     DEFAULTS = ['', '', None, '', '', 0]
 
-    def __init__(self, album_id=None, genre_ids=None):
+    def __init__(self, album_id=None, genre_ids=[]):
         """
             Init album
             @param album_id as int
@@ -208,6 +209,20 @@ class Track(Base):
         self.id = track_id
         self._uri = None
 
+    def formated_name(self):
+        """
+            Return formated name (title \n artist)
+            @return str
+        """
+        name = escape(self.name)
+        if self.album.artist_id == Type.COMPILATIONS or\
+           len(self.artist_ids) > 1 or\
+           self.album.artist_id not in self.artist_ids:
+            if self.artist_names != self.album.artist_name:
+                name = "%s [<b>%s</b>]" % (name,
+                                           escape(self.artist_names))
+        return name
+
     @property
     def title(self):
         """
