@@ -12,7 +12,7 @@
 
 from gi.repository import GObject
 
-from lollypop.define import PlayContext, Lp
+from lollypop.define import PlayContext, Lp, NextContext
 from lollypop.objects import Track
 
 
@@ -22,6 +22,7 @@ class BasePlayer(GObject.GObject):
     """
     __gsignals__ = {
         'current-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
+        'album-added': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         'next-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'prev-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'seeked': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
@@ -41,8 +42,7 @@ class BasePlayer(GObject.GObject):
             GObject.GObject.__init__(self)
             self._base_init = True
             # Keep track of artist/album finished
-            self._album_finished = False
-            self._artist_finished = False
+            self._finished = NextContext.NONE
             # A user playlist used as current playlist
             self._user_playlist = None
             # Used by shuffle tracks to restore user playlist before shuffle
@@ -51,8 +51,10 @@ class BasePlayer(GObject.GObject):
             self.next_track = Track()
             self.prev_track = Track()
             self.context = PlayContext()
+            self.context.next_track = Track()
+            self.context.prev_track = Track()
             # Albums in current playlist
-            self._albums = None
+            self._albums = []
             # Current shuffle mode
             self._shuffle = Lp().settings.get_enum('shuffle')
             # For tracks from the cmd line
