@@ -17,7 +17,7 @@ from time import time
 
 from lollypop.player_base import BasePlayer
 from lollypop.tagreader import ScannerTagReader
-from lollypop.player_rg import ReplayGainPlayer
+from lollypop.player_plugins import PluginsPlayer
 from lollypop.define import GstPlayFlags, NextContext, Lp
 from lollypop.codecs import Codecs
 from lollypop.define import Type
@@ -36,14 +36,12 @@ class BinPlayer(BasePlayer):
         Gst.init(None)
         BasePlayer.__init__(self)
         self._codecs = Codecs()
-        self._gst_duration = 0
         self._crossfading = False
         self._playbin = self._playbin1 = Gst.ElementFactory.make(
                                                            'playbin', 'player')
         self._playbin2 = Gst.ElementFactory.make('playbin', 'player')
-        self._rg1 = ReplayGainPlayer(self._playbin1)
-        self._rg2 = ReplayGainPlayer(self._playbin2)
-        self._volume = 1.0
+        self._plugins = self.plugins1 = PluginsPlayer(self._playbin1)
+        self.plugins2 = PluginsPlayer(self._playbin2)
         self._volume_id = self._playbin.connect('notify::volume',
                                                 self._on_volume_changed)
         for playbin in [self._playbin1, self._playbin2]:
@@ -126,7 +124,6 @@ class BinPlayer(BasePlayer):
         """
             Change player state to STOPPED
         """
-        self._gst_duration = 0
         self._playbin.set_state(Gst.State.NULL)
         self.emit("status-changed")
 
@@ -134,17 +131,6 @@ class BinPlayer(BasePlayer):
         """
             Stop all bins, lollypop should quit now
         """
-        self._gst_duration = 0
-        # Stop crossfade
-        if self._playbin == self._playbin2:
-            self._playbin = self._playbin1
-        else:
-            self._playbin = self._playbin2
-        # Restore volume
-        self._playbin1.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                  self._volume)
-        self._playbin2.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                  self._volume)
         # Stop
         self._playbin1.set_state(Gst.State.NULL)
         self._playbin2.set_state(Gst.State.NULL)
@@ -183,8 +169,8 @@ class BinPlayer(BasePlayer):
             @return position as int
         """
         position = self._playbin.query_position(Gst.Format.TIME)[1] / 1000
-        if self._crossfading and self._gst_duration > 0:
-            duration = (self._gst_duration - position) / 1000000
+        if self._crossfading and self.current_track.duration > 0:
+            duration = self.current_track.duration - position / 1000000
             if duration < Lp().settings.get_value('mix-duration').get_int32():
                 self._do_crossfade(duration)
         return position * 60
@@ -201,7 +187,6 @@ class BinPlayer(BasePlayer):
             Set player volume rate
             @param rate as double
         """
-        self._volume = rate
         self._playbin1.set_volume(GstAudio.StreamVolumeFormat.CUBIC, rate)
         self._playbin2.set_volume(GstAudio.StreamVolumeFormat.CUBIC, rate)
         self.emit('volume-changed')
@@ -215,140 +200,142 @@ class BinPlayer(BasePlayer):
 #######################
 # PRIVATE             #
 #######################
-    def _load(self, track):
+    def _load(self, track, init_volume=True):
         """
             Stop current track, load track id and play it
+            If was playing, do not use play as status doesn't changed
             @param track as Track
+            @param init volume as bool
         """
         was_playing = self.is_playing()
         self._playbin.set_state(Gst.State.NULL)
-        if self._load_track(track):
+        if self._load_track(track, init_volume):
             if was_playing:
                 self._playbin.set_state(Gst.State.PLAYING)
             else:
                 self.play()
 
-    def _volume_up(self, playbin, duration):
+    def _volume_up(self, playbin, plugins, duration):
         """
             Make volume going up smoothly
             @param playbin as Gst.Bin
+            @param plugins as PluginsPlayer
             @param duration as int
         """
         # We are not the active playbin, stop all
         if self._playbin != playbin:
             return
         if duration > 0:
-            vol = playbin.get_volume(GstAudio.StreamVolumeFormat.LINEAR)
+            vol = plugins.volume.props.volume
             steps = duration / 0.25
-            vol_up = (self._volume - vol) / steps
+            vol_up = (1.0 - vol) / steps
             rate = vol + vol_up
-            if rate < self._volume:
-                playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR, rate)
+            if rate < 1.0:
+                plugins.volume.props.volume = rate
                 GLib.timeout_add(250, self._volume_up,
-                                 playbin, duration - 0.25)
+                                 playbin, plugins, duration - 0.25)
             else:
-                playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                   self._volume)
-                if self._volume_id is None:
-                    self._volume_id = playbin.connect('notify::volume',
-                                                      self._on_volume_changed)
+                plugins.volume.props.volume = 1.0
+        else:
+            plugins.volume.props.volume = 1.0
 
-    def _volume_down(self, playbin, duration):
+    def _volume_down(self, playbin, plugins, duration):
         """
             Make volume going down smoothly
             @param playbin as Gst.Bin
+            @param plugins as PluginsPlayer
             @param duration as int
         """
         # We are again the active playbin, stop all
         if self._playbin == playbin:
             return
         if duration > 0:
-            vol = playbin.get_volume(GstAudio.StreamVolumeFormat.LINEAR)
+            vol = plugins.volume.props.volume
             steps = duration / 0.25
             vol_down = vol / steps
             rate = vol - vol_down
             if rate > 0:
-                playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR, rate)
+                plugins.volume.props.volume = rate
                 GLib.timeout_add(250, self._volume_down,
-                                 playbin, duration - 0.25)
+                                 playbin, plugins, duration - 0.25)
             else:
+                plugins.volume.props.volume = 0.0
                 playbin.set_state(Gst.State.NULL)
-                playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                   self._volume)
+        else:
+            plugins.volume.props.volume = 0.0
+            playbin.set_state(Gst.State.NULL)
 
     def _do_crossfade(self, duration, track=None, next=True):
         """
             Crossfade tracks
             @param duration as int
-            @param next as bool
             @param track as Track
+            @param next as bool
         """
-        if self.current_track.id == Type.RADIOS:
+        # No cossfading if we need to stop
+        if self._need_to_stop() and next:
             return
-        if self._volume_id is not None:
-            self._playbin.disconnect(self._volume_id)
-            self._volume_id = None
-        GLib.idle_add(self._volume_down, self._playbin, duration)
+        if self._playbin.query_position(
+           Gst.Format.TIME)[1] / 1000000000 > self.current_track.duration - 10:
+            self._track_finished(self.current_track, self._start_time)
+        GLib.idle_add(self._volume_down, self._playbin,
+                      self._plugins, duration)
         if self._playbin == self._playbin2:
             self._playbin = self._playbin1
+            self._plugins = self.plugins1
         else:
             self._playbin = self._playbin2
+            self._plugins = self.plugins2
 
-        finished = self.current_track
-        finished_start_time = self._start_time
         if track is not None:
-            self._load(track)
-            self._playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                     0)
-            GLib.idle_add(self._volume_up, self._playbin, duration)
+            self._load(track, False)
+            self._plugins.volume.props.volume = 0
+            GLib.idle_add(self._volume_up, self._playbin,
+                          self._plugins, duration)
         elif next and self.next_track.id is not None:
-            self._load(self.next_track)
-            self._playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                     0)
-            GLib.idle_add(self._volume_up, self._playbin, duration)
+            self._load(self.next_track, False)
+            self._plugins.volume.props.volume = 0
+            GLib.idle_add(self._volume_up, self._playbin,
+                          self._plugins, duration)
         elif self.prev_track.id is not None:
-            self._load(self.prev_track)
-            self._playbin.set_volume(GstAudio.StreamVolumeFormat.LINEAR,
-                                     0)
-            GLib.idle_add(self._volume_up, self._playbin, duration)
-        self._track_finished(finished, finished_start_time)
+            self._load(self.prev_track, False)
+            self._plugins.volume.props.volume = 0
+            GLib.idle_add(self._volume_up, self._playbin,
+                          self._plugins, duration)
 
-    def _load_track(self, track):
+    def _need_to_stop(self):
+        """
+            Return True if playback needs to stop
+            @return bool
+        """
+        stop = False
+        if self.context.next != NextContext.NONE:
+            # Stop if needed
+            if self.context.next == NextContext.STOP_TRACK:
+                stop = True
+            elif self.context.next == self._finished:
+                stop = True
+        return stop and self.is_playing()
+
+    def _load_track(self, track, init_volume=True):
         """
             Load track
             @param track as Track
+            @param init volume as bool
             @return False if track not loaded
         """
-        stop = False
-        # Stop if needed
-        if self.context.next == NextContext.STOP_TRACK:
-            stop = True
-
-        # Stop if album changed
-        if self.context.next == NextContext.STOP_ALBUM and\
-           self._album_finished:
-            stop = True
-
-        # Stop if album_artist changed
-        if self.context.next == NextContext.STOP_ARTIST and (
-           self.current_track.album_artist_id != track.album_artist_id or
-           self._artist_finished):
-            stop = True
-
-        if stop and self.is_playing():
+        if self._need_to_stop():
             return False
-
-        self._album_finished = False
-        self._artist_finished = False
+        if init_volume:
+            self._plugins.volume.props.volume = 1.0
+        debug("BinPlayer::_load_track(): %s" % track.uri)
         self.current_track = track
-
         try:
             self._playbin.set_property('uri',
                                        self.current_track.uri)
         except Exception as e:  # Gstreamer error
             print("BinPlayer::_load_track(): ", e)
             return False
-
         return True
 
     def _track_finished(self, finished, finished_start_time):
@@ -380,10 +367,13 @@ class BinPlayer(BasePlayer):
             @param playbin as Gst.Bin
             @param sink as Gst.Sink
         """
-        volume = playbin.get_volume(GstAudio.StreamVolumeFormat.LINEAR)
-        if volume != self._volume:
-            self._volume = volume
-            self.emit('volume-changed')
+        if playbin == self._playbin1:
+            vol = self._playbin1.get_volume(GstAudio.StreamVolumeFormat.CUBIC)
+            self._playbin2.set_volume(GstAudio.StreamVolumeFormat.CUBIC, vol)
+        else:
+            vol = self._playbin2.get_volume(GstAudio.StreamVolumeFormat.CUBIC)
+            self._playbin1.set_volume(GstAudio.StreamVolumeFormat.CUBIC, vol)
+        self.emit('volume-changed')
 
     def _on_bus_message_tag(self, bus, message):
         """
@@ -458,27 +448,26 @@ class BinPlayer(BasePlayer):
 
     def _on_bus_eos(self, bus, message):
         """
-            On end of stream, stop playing if user ask for,
+            On end of stream, stop playback
             go next otherwise
         """
         debug("Player::_on_bus_eos(): %s" % self.current_track.uri)
-        if self.context.next not in [NextContext.NONE,
-                                     NextContext.START_NEW_ALBUM]:
+        if self._playbin.get_bus() == bus:
             self.stop()
+            self._finished = NextContext.NONE
             self.context.next = NextContext.NONE
             if self.next_track.id is not None:
                 self._load_track(self.next_track)
             self.emit('current-changed')
-        elif not Lp().settings.get_value('mix'):
-            self.next()
 
     def _on_stream_about_to_finish(self, playbin):
         """
             When stream is about to finish, switch to next track without gap
             @param playbin as Gst bin
         """
+        debug("Player::_on_stream_about_to_finish(): %s" % playbin)
         # Don't do anything if crossfade on
-        if self._playbin != playbin:
+        if self._crossfading:
             return
         if self.current_track.id == Type.RADIOS:
             return
@@ -496,8 +485,6 @@ class BinPlayer(BasePlayer):
             @param message as Gst.Message
         """
         self._start_time = time()
-        self._gst_duration = self._playbin.query_duration(
-                                                    Gst.Format.TIME)[1] / 1000
         debug("Player::_on_stream_start(): %s" % self.current_track.uri)
         self.emit('current-changed')
         # Update now playing on lastfm
