@@ -36,8 +36,10 @@ class LinearPlayer(BasePlayer):
             return self.current_track
 
         track = Track()
-        if self._albums:
-            album = Album(self.current_track.album.id, self.context.genre_ids)
+        if self.current_track.album.id in self.context.genre_ids and \
+                self._albums:
+            genre_ids = self.context.genre_ids[self.current_track.album.id]
+            album = Album(self.current_track.album.id, genre_ids)
             if self.current_track.id in album.tracks_ids:
                 new_track_position = album.tracks_ids.index(
                                                 self.current_track.id) + 1
@@ -47,18 +49,19 @@ class LinearPlayer(BasePlayer):
                     if self.context.next == NextContext.START_NEW_ALBUM:
                         self.context.next = NextContext.NONE
                     try:
-                        self._album_finished = True
+                        self._finished = NextContext.STOP_ALBUM
                         pos = self._albums.index(album.id)
                         # we are on last album, go to first
                         if pos + 1 >= len(self._albums):
-                            self._artist_finished = True
+                            self._finished = NextContext.STOP_ALL
                             pos = 0
                         else:
                             pos += 1
                     except:
                         pos = 0  # Happens if current album has been removed
+                    genre_ids = self.context.genre_ids[self._albums[pos]]
                     track = Album(self._albums[pos],
-                                  self.context.genre_ids).tracks[0]
+                                  genre_ids).tracks[0]
                 # next track
                 else:
                     track = album.tracks[new_track_position]
@@ -74,8 +77,10 @@ class LinearPlayer(BasePlayer):
             return self.current_track
 
         track = Track()
-        if self._albums:
-            album = Album(self.current_track.album.id, self.context.genre_ids)
+        if self.current_track.album.id in self.context.genre_ids and \
+                self._albums:
+            genre_ids = self.context.genre_ids[self.current_track.album.id]
+            album = Album(self.current_track.album.id, genre_ids)
             if self.current_track.id in album.tracks_ids:
                 new_track_position = album.tracks_ids.index(
                                                 self.current_track.id) - 1
@@ -89,8 +94,9 @@ class LinearPlayer(BasePlayer):
                             pos -= 1
                     except:
                         pos = 0  # Happens if current album has been removed
+                    genre_ids = self.context.genre_ids[self._albums[pos]]
                     track = Album(self._albums[pos],
-                                  self.context.genre_ids).tracks[-1]
+                                  genre_ids).tracks[-1]
                 # Previous track
                 else:
                     track = album.tracks[new_track_position]
