@@ -24,48 +24,63 @@ class QueuePlayer:
         """
         self._queue = []
 
-    def append_to_queue(self, track_id):
+    def append_to_queue(self, track_id, notify=True):
         """
             Append track to queue,
             remove previous track if exist
             @param track id as int
+            @param notify as bool
         """
         if track_id in self._queue:
             self._queue.remove(track_id)
         self._queue.append(track_id)
-        self.set_next()
-        self.emit("queue-changed")
+        if not self._init_current_if_needed():
+            self.set_next()
+            self.set_prev()
+        if notify:
+            self.emit('queue-changed')
 
-    def prepend_to_queue(self, track_id):
+    def insert_in_queue(self, track_id, pos=0, notify=True):
         """
             Prepend track to queue,
             remove previous track if exist
             @param track id as int
+            @param notify as bool
         """
         if track_id in self._queue:
             self._queue.remove(track_id)
-        self._queue.insert(0, track_id)
-        self.set_next()
-        self.emit("queue-changed")
+        self._queue.insert(pos, track_id)
+        if not self._init_current_if_needed():
+            self.set_next()
+            self.set_prev()
+        if notify:
+            self.emit('queue-changed')
 
-    def del_from_queue(self, track_id):
+    def del_from_queue(self, track_id, notify=True):
         """
             Remove track from queue
             @param track id as int
+            @param notify as bool
         """
         if track_id in self._queue:
             self._queue.remove(track_id)
             self.set_next()
-            self.emit("queue-changed")
+            self.set_prev()
+        if notify:
+            self.emit('queue-changed')
 
-    def set_queue(self, new_queue):
+    def set_queue(self, new_queue, notify=True):
         """
             Set queue
             @param [ids as int]
+            @param notify as bool
         """
         self._queue = new_queue
-        self.set_next()
-        self.emit("queue-changed")
+        if not self._init_current_if_needed():
+            self.set_next()
+            self.set_prev()
+        if notify:
+            self.emit('queue-changed')
 
     def get_queue(self):
         """
@@ -109,3 +124,16 @@ class QueuePlayer:
 #######################
 # PRIVATE             #
 #######################
+    def _init_current_if_needed(self):
+        """
+            Init current track if needed
+        """
+        if self.current_track.id is None and self._queue:
+            self._load_track(QueuePlayer.next(self))
+            if self.current_track.id in self._queue:
+                self._queue.remove(self.current_track.id)
+            self.set_next()
+            self.set_prev()
+            self.emit('current-changed')
+            return True
+        return False
