@@ -53,7 +53,6 @@ class RadioPlayer(BasePlayer):
                 self.set_party(False)
             self.next_track = Track()
             self.emit('next-changed')
-            self._albums = None
 
     def next(self):
         """
@@ -120,12 +119,13 @@ class RadioPlayer(BasePlayer):
             Start playing track
             @param track as Track:
         """
+        self._plugins.volume.props.volume = 1.0
         self._playbin.set_state(Gst.State.NULL)
         self._playbin.set_property('uri', track.uri)
         Radios().set_more_popular(track.album_artist)
         self.current_track = track
         self._current = None
-        self.play()
+        self._playbin.set_state(Gst.State.PLAYING)
 
     def _on_parse_finished(self, parser, result, track):
         """
