@@ -46,8 +46,9 @@ class ShufflePlayer(BasePlayer):
         self._already_played_albums = []
         # Tracks already played for albums
         self._already_played_tracks = {}
-        # Reset use playlist
+        # Reset user playlist
         self._user_playlist = []
+        self._user_playlist_ids = []
 
     def next(self):
         """
@@ -97,29 +98,40 @@ class ShufflePlayer(BasePlayer):
             Play a new random track if not already playing
             @param party as bool
         """
-        self.reset_history()
+        if party == self._is_party:
+            return
+        self._is_party = party
 
-        if self._rg1.rgvolume is not None and self._rg2.rgvolume is not None:
+        self.reset_history()
+        self.context.genre_ids = {}
+
+        if self.plugins1.rgvolume is not None and\
+           self.plugins2.rgvolume is not None:
             if party:
                 self.context.next = NextContext.NONE
-                self._rg1.rgvolume.props.album_mode = 0
-                self._rg2.rgvolume.props.album_mode = 0
+                self.plugins1.rgvolume.props.album_mode = 0
+                self.plugins2.rgvolume.props.album_mode = 0
             else:
-                self._rg1.rgvolume.props.album_mode = 1
-                self._rg2.rgvolume.props.album_mode = 1
-
-        self._is_party = party
+                self.plugins1.rgvolume.props.album_mode = 1
+                self.plugins2.rgvolume.props.album_mode = 1
 
         if party:
             self._external_tracks = []
-            self.context.genre_ids = []
+            self.context.genre_ids = {}
             self.context.track_id = None
             party_ids = self.get_party_ids()
             if party_ids:
                 self._albums = Lp().albums.get_party_ids(party_ids)
             else:
                 self._albums = Lp().albums.get_ids()
-
+            # We do not store genre_ids for ALL/POPULARS/...
+            genre_ids = []
+            for genre_id in party_ids:
+                if genre_id > 0:
+                    genre_ids.append(genre_id)
+            # Set context for each album
+            for album_id in self._albums:
+                self.context.genre_ids[album_id] = genre_ids
             # Start a new song if not playing
             if (self.current_track.id in [None, Type.RADIOS])\
                     and self._albums:
@@ -141,6 +153,23 @@ class ShufflePlayer(BasePlayer):
         """
         return self._is_party
 
+    def shuffle_albums(self, shuffle):
+        """
+            Shuffle album list
+            @param shuffle as bool
+        """
+        if shuffle and self._shuffle == Shuffle.ALBUMS:
+            if self._albums:
+                self._albums_backup = list(self._albums)
+                random.shuffle(self._albums)
+                # In album shuffle, keep current album on top
+                if self.current_track.album.id in self._albums:
+                    self._albums.remove(self.current_track.album.id)
+                    self._albums.insert(0, self.current_track.album.id)
+        elif self._albums_backup:
+            self._albums = self._albums_backup
+            self._albums_backup = []
+
 #######################
 # PRIVATE             #
 #######################
@@ -151,36 +180,24 @@ class ShufflePlayer(BasePlayer):
         """
         self._shuffle = Lp().settings.get_enum('shuffle')
 
-        if self._rg1.rgvolume is not None and self._rg2.rgvolume is not None:
+        if self.plugins1.rgvolume is not None and\
+           self.plugins2.rgvolume is not None:
             if self._shuffle == Shuffle.TRACKS or self._user_playlist:
-                self._rg1.rgvolume.props.album_mode = 0
-                self._rg2.rgvolume.props.album_mode = 0
+                self.plugins1.rgvolume.props.album_mode = 0
+                self.plugins2.rgvolume.props.album_mode = 0
             else:
-                self._rg1.rgvolume.props.album_mode = 1
-                self._rg2.rgvolume.props.album_mode = 1
+                self.plugins1.rgvolume.props.album_mode = 1
+                self.plugins2.rgvolume.props.album_mode = 1
 
         if self._user_playlist:
             self._shuffle_playlist()
-        elif self.current_track.id is not None and self.current_track.id >= 0:
-            self.set_albums(self.current_track.id,
-                            self.context.artist_ids,
-                            self.context.genre_ids)
+        elif self._shuffle == Shuffle.NONE:
+            self.shuffle_albums(False)
+        elif self._shuffle == Shuffle.ALBUMS:
+            self.shuffle_albums(True)
         if self.current_track.id is not None:
             self.set_next()
 
-    def _shuffle_albums(self):
-        """
-            Shuffle album list
-        """
-        if self._shuffle == Shuffle.ALBUMS:
-            if self._albums and not self._albums_backup:
-                self._albums_backup = list(self._albums)
-                random.shuffle(self._albums)
-        elif self._shuffle == Shuffle.NONE:
-            if self._albums_backup:
-                self._albums = self._albums_backup
-                self._albums_backup = []
-
     def _shuffle_next(self):
         """
             Next track in shuffle mode
@@ -203,11 +220,18 @@ class ShufflePlayer(BasePlayer):
         """
         for album_id in sorted(self._albums,
                                key=lambda *args: random.random()):
-            tracks = Album(album_id, self.context.genre_ids).tracks_ids
+            # We need to check this as in party mode, some items do not
+            # have a valid genre (Populars, ...)
+            if album_id in self.context.genre_ids.keys():
+                genre_ids = self.context.genre_ids[album_id]
+            else:
+                genre_ids = []
+            tracks = Album(album_id, genre_ids).tracks_ids
             for track in sorted(tracks, key=lambda *args: random.random()):
                 if album_id not in self._already_played_tracks.keys() or\
                    track not in self._already_played_tracks[album_id]:
                     return track
+            self._finished = NextContext.STOP_ALBUM
             # No new tracks for this album, remove it
             # If albums not in shuffle history, it's not present
             # in db anymore (update since shuffle set)
@@ -215,6 +239,7 @@ class ShufflePlayer(BasePlayer):
                 self._already_played_tracks.pop(album_id)
                 self._already_played_albums.append(album_id)
             self._albums.remove(album_id)
+        self._finished = NextContext.STOP_ALL
         return None
 
     def _add_to_shuffle_history(self, track):
