@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GObject, GLib, Gio, TotemPlParser
+from gi.repository import GObject, GLib
 
 import os
 from gettext import gettext as _
@@ -34,7 +34,8 @@ class Playlists(GObject.GObject):
         # Add or remove a playlist
         'playlists-changed': (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         # Objects added/removed to/from playlist
-        'playlist-changed': (GObject.SignalFlags.RUN_FIRST, None, (int,))
+        'playlist-add': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
+        'playlist-del': (GObject.SignalFlags.RUN_FIRST, None, (int, int))
     }
     create_playlists = '''CREATE TABLE playlists (
                             id INTEGER PRIMARY KEY,
@@ -51,8 +52,6 @@ class Playlists(GObject.GObject):
         """
         GObject.GObject.__init__(self)
         self._LOVED = _("Loved tracks")
-        self._MPD = _("Network control")
-        try_import = not os.path.exists(self.DB_PATH)
         # Create db schema
         try:
             with SqlCursor(self) as sql:
@@ -62,29 +61,6 @@ class Playlists(GObject.GObject):
         except:
             pass
 
-        # We import playlists from lollypop < 0.9.60
-        if try_import:
-            try:
-                d = Gio.File.new_for_path(self.LOCAL_PATH + "/playlists")
-                infos = d.enumerate_children(
-                    'standard::name',
-                    Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
-                    None)
-                for info in infos:
-                    f = info.get_name()
-                    if f.endswith(".m3u"):
-                        if f[:-4] != self._LOVED:
-                            self.add(f[:-4])
-                        playlist_id = self.get_id(f[:-4])
-                        parser = TotemPlParser.Parser.new()
-                        parser.connect('entry-parsed',
-                                       self._on_entry_parsed,
-                                       playlist_id)
-                        parser.parse_async(d.get_uri() + "/%s" % f,
-                                           True, None, None)
-            except:
-                pass
-
     def add(self, name):
         """
             Add a playlist
@@ -213,8 +189,6 @@ class Playlists(GObject.GObject):
         """
         if playlist_name == self._LOVED:
             return Type.LOVED
-        elif playlist_name == self._MPD:
-            return Type.MPD
 
         with SqlCursor(self) as sql:
             result = sql.execute("SELECT rowid\
@@ -233,8 +207,6 @@ class Playlists(GObject.GObject):
         """
         if playlist_id == Type.LOVED:
             return self._LOVED
-        elif playlist_id == Type.MPD:
-            return self._MPD
 
         with SqlCursor(self) as sql:
             result = sql.execute("SELECT name\
@@ -256,7 +228,7 @@ class Playlists(GObject.GObject):
                          WHERE playlist_id=?", (playlist_id,))
             sql.commit()
             if notify:
-                GLib.idle_add(self.emit, 'playlist-changed', playlist_id)
+                GLib.idle_add(self.emit, 'playlist-del', playlist_id, None)
 
     def add_tracks(self, playlist_id, tracks, notify=True):
         """
@@ -273,13 +245,14 @@ class Playlists(GObject.GObject):
                     sql.execute("INSERT INTO tracks"
                                 " VALUES (?, ?)",
                                 (playlist_id, track.path))
+                if notify:
+                    GLib.idle_add(self.emit, 'playlist-add',
+                                  playlist_id, track.id)
             if changed:
                 sql.execute("UPDATE playlists SET mtime=?\
                              WHERE rowid=?", (datetime.now().strftime('%s'),
                                               playlist_id))
                 sql.commit()
-                if notify:
-                    GLib.idle_add(self.emit, 'playlist-changed', playlist_id)
 
     def remove_tracks(self, playlist_id, tracks, notify=True):
         """
@@ -292,9 +265,10 @@ class Playlists(GObject.GObject):
                 sql.execute("DELETE FROM tracks\
                              WHERE filepath=?\
                              AND playlist_id=?", (track.path, playlist_id))
+                if notify:
+                    GLib.idle_add(self.emit, 'playlist-del',
+                                  playlist_id, track.id)
             sql.commit()
-            if notify:
-                GLib.idle_add(self.emit, 'playlist-changed', playlist_id)
 
     def get_position(self, playlist_id, track_id):
         """
