@@ -18,7 +18,7 @@ from threading import Thread
 
 from lollypop.widgets_rating import RatingWidget
 from lollypop.widgets_loved import LovedWidget
-from lollypop.define import Lp, NextContext
+from lollypop.define import Lp, NextContext, Shuffle
 from lollypop.objects import Track
 from lollypop import utils
 
@@ -61,27 +61,42 @@ class PlaybackMenu(BaseMenu):
         """
             Set playback actions
         """
-        if Lp().player.context.next != NextContext.NONE:
-            continue_play_action = Gio.SimpleAction(
-                name="continue_play_action")
-            self._app.add_action(continue_play_action)
-            continue_play_action.connect('activate', self._continue_playback)
-            self.append(_("Continue playback"), 'app.continue_play_action')
-        if Lp().player.context.next != NextContext.STOP_TRACK:
-            stop_track_action = Gio.SimpleAction(name="stop_track_action")
-            self._app.add_action(stop_track_action)
-            stop_track_action.connect('activate', self._stop_track)
-            self.append(_("Stop after this track"), 'app.stop_track_action')
-        if Lp().player.context.next != NextContext.STOP_ALBUM:
-            stop_album_action = Gio.SimpleAction(name="stop_album_action")
-            self._app.add_action(stop_album_action)
-            stop_album_action.connect('activate', self._stop_album)
-            self.append(_("Stop after this album"), 'app.stop_album_action')
-        if Lp().player.context.next != NextContext.STOP_ARTIST:
-            stop_artist_action = Gio.SimpleAction(name="stop_artist_action")
-            self._app.add_action(stop_artist_action)
-            stop_artist_action.connect('activate', self._stop_artist)
-            self.append(_("Stop after this artist"), 'app.stop_artist_action')
+        shuffle = Lp().settings.get_enum('shuffle')
+        if shuffle != Shuffle.TRACKS:
+            # Reset context action and nothing more
+            if Lp().player.context.next != NextContext.NONE:
+                continue_play_action = Gio.SimpleAction(
+                    name="continue_play_action")
+                self._app.add_action(continue_play_action)
+                continue_play_action.connect('activate',
+                                             self._continue_playback)
+                self.append(_("Continue playback"), 'app.continue_play_action')
+                return
+            if not Lp().player.get_queue() and\
+               not Lp().player.get_user_playlist_ids():
+                # Stop after track
+                if Lp().player.context.next != NextContext.STOP_TRACK:
+                    stop_track_action = Gio.SimpleAction(
+                                                      name="stop_track_action")
+                    self._app.add_action(stop_track_action)
+                    stop_track_action.connect('activate', self._stop_track)
+                    self.append(_("Stop after this track"),
+                                'app.stop_track_action')
+                # Stop after album
+                if Lp().player.context.next != NextContext.STOP_ALBUM:
+                    stop_album_action = Gio.SimpleAction(
+                                                      name="stop_album_action")
+                    self._app.add_action(stop_album_action)
+                    stop_album_action.connect('activate', self._stop_album)
+                    self.append(_("Stop after this album"),
+                                'app.stop_album_action')
+        # Do not repeat
+        if Lp().player.context.next != NextContext.STOP_ALL:
+            do_not_repeat_action = Gio.SimpleAction(
+                                                   name="do_not_repeat_action")
+            self._app.add_action(do_not_repeat_action)
+            do_not_repeat_action.connect('activate', self._do_not_repeat)
+            self.append(_("Do not repeat"), 'app.do_not_repeat_action')
 
     def _continue_playback(self, action, variant):
         """
@@ -92,6 +107,14 @@ class PlaybackMenu(BaseMenu):
         """
         Lp().player.context.next = NextContext.NONE
 
+    def _stop_playlist(self, action, variant):
+        """
+            Tell player to stop after current playlist
+            @param SimpleAction
+            @param GVariant
+        """
+        Lp().player.context.next = NextContext.STOP_PLAYLIST
+
     def _stop_track(self, action, variant):
         """
             Tell player to stop after current track
@@ -108,13 +131,13 @@ class PlaybackMenu(BaseMenu):
         """
         Lp().player.context.next = NextContext.STOP_ALBUM
 
-    def _stop_artist(self, action, variant):
+    def _do_not_repeat(self, action, variant):
         """
             Tell player to stop after current artist
             @param SimpleAction
             @param GVariant
         """
-        Lp().player.context.next = NextContext.STOP_ARTIST
+        Lp().player.context.next = NextContext.STOP_ALL
 
 
 class QueueMenu(BaseMenu):
@@ -175,7 +198,7 @@ class QueueMenu(BaseMenu):
             self.append(_("Add to queue"), 'app.append_queue_action')
         if prepend:
             prepend_queue_action.connect('activate',
-                                         self._prepend_to_queue)
+                                         self._insert_in_queue)
             if self._is_album:
                 self.append(_("Next tracks"), 'app.prepend_queue_action')
             else:
@@ -194,11 +217,12 @@ class QueueMenu(BaseMenu):
         if self._is_album:
             for track_id in Lp().albums.get_tracks(self._object_id,
                                                    self._genre_id):
-                Lp().player.append_to_queue(track_id)
+                Lp().player.append_to_queue(track_id, False)
         else:
-            Lp().player.append_to_queue(self._object_id)
+            Lp().player.append_to_queue(self._object_id, False)
+        Lp().player.emit('queue-changed')
 
-    def _prepend_to_queue(self, action, variant):
+    def _insert_in_queue(self, action, variant):
         """
             Prepend track id to queue
             @param SimpleAction
@@ -207,9 +231,10 @@ class QueueMenu(BaseMenu):
         if self._is_album:
             for track_id in reversed(Lp().albums.get_tracks(self._object_id,
                                                             self._genre_id)):
-                Lp().player.prepend_to_queue(track_id)
+                Lp().player.insert_in_queue(track_id, 0, False)
         else:
-            Lp().player.prepend_to_queue(self._object_id)
+            Lp().player.insert_in_queue(self._object_id, 0, False)
+        Lp().player.emit('queue-changed')
 
     def _del_from_queue(self, action, variant):
         """
@@ -220,9 +245,10 @@ class QueueMenu(BaseMenu):
         if self._is_album:
             for track_id in Lp().albums.get_tracks(self._object_id,
                                                    self._genre_id):
-                Lp().player.del_from_queue(track_id)
+                Lp().player.del_from_queue(track_id, False)
         else:
-            Lp().player.del_from_queue(self._object_id)
+            Lp().player.del_from_queue(self._object_id, False)
+        Lp().player.emit('queue-changed')
 
 
 class PlaylistsMenu(BaseMenu):
