@@ -12,7 +12,7 @@
 
 from gi.repository import Gtk
 
-from lollypop.define import Lp, ArtSize
+from lollypop.define import Lp, ArtSize, Shuffle
 
 
 class NextPopover(Gtk.Popover):
@@ -39,11 +39,10 @@ class NextPopover(Gtk.Popover):
         self._artist_label = builder.get_object('artist')
         self._cover = builder.get_object('cover')
         self._skip_btn = builder.get_object('skip_btn')
-        self._signal_id = Lp().player.connect('queue-changed', self.update)
 
-    def update(self, player=None):
+    def update(self, unused=None):
         """
-            Update widget with current track
+            Update widget with next track
         """
         self._artist_label.set_text(Lp().player.next_track.artist)
         self._title_label.set_text(Lp().player.next_track.title)
@@ -55,6 +54,11 @@ class NextPopover(Gtk.Popover):
             del art
             self._cover.set_tooltip_text(Lp().player.next_track.album.name)
             self._cover.show()
+            queue = Lp().player.get_queue()
+            if queue and queue[0] == Lp().player.next_track.id:
+                self._skip_btn.hide()
+            else:
+                self._skip_btn.show()
         else:
             self._cover.hide()
 
@@ -62,7 +66,11 @@ class NextPopover(Gtk.Popover):
         """
             Return True if widget should be shown, not already closed by user
         """
-        return self._should_be_shown
+        return self._should_be_shown and (
+                Lp().player.is_party() or
+                Lp().settings.get_enum('shuffle') == Shuffle.TRACKS) and\
+            Lp().player.next_track.id is not None and\
+            Lp().player.next_track.id >= 0
 
 #######################
 # PRIVATE             #
@@ -82,6 +90,7 @@ class NextPopover(Gtk.Popover):
         """
         if self._signal_id is not None:
             Lp().player.disconnect(self._signal_id)
+            self._signal_id = None
 
     def _on_enter_notify(self, widget, event):
         """
