@@ -26,13 +26,11 @@ class SearchRow(Gtk.ListBoxRow):
         Album/Track search row
     """
 
-    def __init__(self, parent):
+    def __init__(self):
         """
             Init row widgets
-            @param parent as Gtk.Widget
         """
         Gtk.ListBoxRow.__init__(self)
-        self._parent = parent
         self.id = None
         self.is_track = False
         builder = Gtk.Builder()
@@ -41,11 +39,14 @@ class SearchRow(Gtk.ListBoxRow):
         self.set_property('has-tooltip', True)
         self.connect('query-tooltip', self._on_query_tooltip)
         self._row_widget = builder.get_object('row')
+        self._row_widget.set_margin_top(2)
+        self._row_widget.set_margin_end(2)
         self._artist = builder.get_object('artist')
         self._title = builder.get_object('item')
         self._cover = builder.get_object('cover')
         self.add(self._row_widget)
         self.show()
+        self.get_style_context().add_class('trackrow')
 
     def set_text(self, artist, title):
         """
@@ -140,16 +141,14 @@ class SearchPopover(Gtk.Popover):
         Popover allowing user to search for tracks/albums
     """
 
-    def __init__(self, parent):
+    def __init__(self):
         """
             Init Popover
-            @param parent as Gtk.Widget
         """
         Gtk.Popover.__init__(self)
         self.set_position(Gtk.PositionType.BOTTOM)
         self.connect('map', self._on_map)
         self.connect('unmap', self._on_unmap)
-        self._parent = parent
         self._in_thread = False
         self._stop_thread = False
         self._timeout = None
@@ -159,12 +158,12 @@ class SearchPopover(Gtk.Popover):
         builder.add_from_resource('/org/gnome/Lollypop/SearchPopover.ui')
         builder.connect_signals(self)
 
-        self._play_btn = builder.get_object('play_btn')
         self._new_btn = builder.get_object('new_btn')
 
         self._view = Gtk.ListBox()
+        self._view.get_style_context().add_class('trackswidget')
         self._view.connect("button-press-event", self._on_button_press)
-        self._view.connect("row-activated", self._on_activate)
+        self._view.connect("row-activated", self._on_row_activated)
         self._view.show()
 
         builder.get_object('scrolled').add(self._view)
@@ -174,11 +173,8 @@ class SearchPopover(Gtk.Popover):
         """
             Set widget size
         """
-        size_setting = Lp().settings.get_value('window-size')
-        if isinstance(size_setting[1], int):
-            self.set_size_request(400, size_setting[1]*0.7)
-        else:
-            self.set_size_request(400, 600)
+        height = Lp().window.get_size()[1]
+        self.set_size_request(400, height*0.7)
         Gtk.Popover.do_show(self)
 
 #######################
@@ -211,7 +207,7 @@ class SearchPopover(Gtk.Popover):
 
     def _populate(self):
         """
-            Populate treeview searching items
+            Populate searching items
             in db based on text entry current text
         """
         results = []
@@ -228,17 +224,6 @@ class SearchPopover(Gtk.Popover):
                                                         artist_id):
                 tracks_non_album_artist.append((track_id, track_name))
 
-        albums += Lp().albums.search(self._current_search)
-
-        for album_id, artist_id in albums:
-            search_obj = SearchObject()
-            search_obj.artist = Lp().artists.get_name(artist_id)
-            search_obj.title = Lp().albums.get_name(album_id)
-            search_obj.count = Lp().albums.get_count(album_id, None)
-            search_obj.id = album_id
-            search_obj.album_id = album_id
-            results.append(search_obj)
-
         for track_id, track_name in Lp().tracks.search(
                         self._current_search) + tracks_non_album_artist:
             search_obj = SearchObject()
@@ -252,7 +237,17 @@ class SearchPopover(Gtk.Popover):
                 search_obj.artist = Lp().tracks.get_artist_names(track_id)
             else:
                 search_obj.artist = Lp().artists.get_name(artist_id)
+            results.append(search_obj)
 
+        albums += Lp().albums.search(self._current_search)
+
+        for album_id, artist_id in albums:
+            search_obj = SearchObject()
+            search_obj.artist = Lp().artists.get_name(artist_id)
+            search_obj.title = Lp().albums.get_name(album_id)
+            search_obj.count = Lp().albums.get_count(album_id, None)
+            search_obj.id = album_id
+            search_obj.album_id = album_id
             results.append(search_obj)
 
         if not self._stop_thread:
@@ -270,7 +265,7 @@ class SearchPopover(Gtk.Popover):
         if results:
             result = results.pop(0)
             if not self._exists(result):
-                search_row = SearchRow(self._parent)
+                search_row = SearchRow()
                 if result.count != -1:
                     result.title += " (%s)" % result.count
                 search_row.set_text(result.artist, result.title)
@@ -280,7 +275,10 @@ class SearchPopover(Gtk.Popover):
                                      ArtSize.MEDIUM*self.get_scale_factor()))
                 search_row.id = result.id
                 search_row.is_track = result.is_track
-                self._view.add(search_row)
+                if search_row.is_track:
+                    self._view.prepend(search_row)
+                else:
+                    self._view.add(search_row)
             if self._stop_thread:
                 self._in_thread = False
                 self._stop_thread = False
@@ -290,15 +288,15 @@ class SearchPopover(Gtk.Popover):
             self._in_thread = False
             self._stop_thread = False
 
-    def _set_user_playlist_by_tracks(self, tracks, track_id):
+    def _populate_user_playlist_by_tracks(self, track_ids, track_id):
         """
             Set user playlist
-            @param tracks as [Track]
+            @param track_ids as [int]
             @param track id as int
             @thread safe
         """
         Lp().player.load(Track(track_id))
-        Lp().player.set_user_playlist_by_tracks(tracks)
+        Lp().player.populate_user_playlist_by_tracks(track_ids, [Type.SEARCH])
 
     def _play_search(self, object_id=None, is_track=True):
         """
@@ -306,23 +304,24 @@ class SearchPopover(Gtk.Popover):
             @param started object id as int
             @param is track as bool
         """
-        tracks = []
+        track_ids = []
         track_id = None
         for child in self._view.get_children():
             if child.is_track:
-                tracks.append(Track(child.id))
+                track_ids.append(child.id)
             else:
                 album_tracks = Lp().albums.get_tracks(child.id, None)
                 if not is_track and child.id == object_id and album_tracks:
                     track_id = album_tracks[0]
                 for tid in album_tracks:
-                    tracks.append(Track(tid))
-        if tracks:
+                    track_ids.append(tid)
+        if track_ids:
             if object_id is not None and is_track:
                 track_id = object_id
             elif track_id is None:
-                track_id = tracks[0].id
-            GLib.idle_add(self._set_user_playlist_by_tracks, tracks, track_id)
+                track_id = track_ids[0]
+            GLib.idle_add(self._populate_user_playlist_by_tracks,
+                          track_ids, track_id)
 
     def _new_playlist(self):
         """
@@ -372,12 +371,10 @@ class SearchPopover(Gtk.Popover):
 
         self._current_search = widget.get_text()
         if self._current_search != "":
-            self._play_btn.set_sensitive(True)
             self._new_btn.set_sensitive(True)
             self._timeout = GLib.timeout_add(100,
                                              self._on_search_changed_thread)
         else:
-            self._play_btn.set_sensitive(False)
             self._new_btn.set_sensitive(False)
             self._clear([])
 
@@ -391,15 +388,6 @@ class SearchPopover(Gtk.Popover):
         t.daemon = True
         t.start()
 
-    def _on_play_btn_clicked(self, button):
-        """
-            Start playback base on current search
-            @param button as Gtk.Button
-        """
-        t = Thread(target=self._play_search)
-        t.daemon = True
-        t.start()
-
     def _on_new_btn_clicked(self, button):
         """
             Create a new playlist based on search
@@ -409,7 +397,7 @@ class SearchPopover(Gtk.Popover):
         t.daemon = True
         t.start()
 
-    def _on_activate(self, widget, row):
+    def _on_row_activated(self, widget, row):
         """
             Play searched item when selected
             @param widget as Gtk.ListBox
@@ -418,10 +406,15 @@ class SearchPopover(Gtk.Popover):
         if Lp().player.is_party():
             if row.is_track:
                 Lp().player.load(Track(row.id))
-            else:
-                popover = AlbumPopoverWidget(row.id, None)
+            elif Gtk.get_minor_version() > 16:
+                popover = AlbumPopoverWidget(row.id, [], [])
                 popover.set_relative_to(row)
                 popover.show()
+            else:
+                t = Thread(target=self._play_search, args=(row.id,
+                                                           row.is_track))
+                t.daemon = True
+                t.start()
         else:
             t = Thread(target=self._play_search, args=(row.id, row.is_track))
             t.daemon = True
@@ -445,7 +438,7 @@ class SearchPopover(Gtk.Popover):
                 popover.set_pointing_to(rect)
                 popover.show()
             else:
-                popover = AlbumPopoverWidget(row.id, None)
+                popover = AlbumPopoverWidget(row.id, [], [])
                 popover.set_relative_to(widget)
                 popover.set_pointing_to(rect)
                 popover.show()
