@@ -13,8 +13,9 @@
 from gi.repository import Gtk, Gdk, GLib, GObject, Pango
 
 from cgi import escape
+from gettext import gettext as _
 
-from lollypop.define import Type, Lp
+from lollypop.define import Type, Lp, SelectionMode
 
 
 class SelectionPopover(Gtk.Popover):
@@ -68,13 +69,15 @@ class SelectionList(Gtk.ScrolledWindow):
         'populated': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
-    def __init__(self):
+    def __init__(self, mode):
         """
             Init Selection list ui
+            @param mode as SelectionMode
         """
         Gtk.ScrolledWindow.__init__(self)
         self.set_policy(Gtk.PolicyType.NEVER,
                         Gtk.PolicyType.AUTOMATIC)
+        self._mode = mode
         self._last_motion_event = None
         self._previous_motion_y = 0.0
         self._timeout = None
@@ -138,12 +141,6 @@ class SelectionList(Gtk.ScrolledWindow):
         self.emit('populated')
         self._updating = False
 
-    def set_mode(self, mode):
-        """
-            Set selection mode
-        """
-        self._selection.set_mode(mode)
-
     def remove(self, object_id):
         """
             Remove row from model
@@ -272,8 +269,12 @@ class SelectionList(Gtk.ScrolledWindow):
             @param value as [int, str]
             @thread safe
         """
+        if value[1] == "":
+            string = _("Unknown")
+        else:
+            string = value[1]
         i = self._model.append([value[0],
-                                value[1],
+                                string,
                                 self._get_icon_name(value[0])])
         if value[0] in self._to_select_ids:
             self._to_select_ids.remove(value[0])
@@ -320,8 +321,6 @@ class SelectionList(Gtk.ScrolledWindow):
             icon = 'emblem-favorite-symbolic'
         elif object_id == Type.NEVER:
             icon = 'document-new-symbolic'
-        elif object_id == Type.MPD:
-            icon = 'network-workgroup-symbolic'
         return icon
 
     def _sort_items(self, model, itera, iterb, data):
@@ -363,7 +362,7 @@ class SelectionList(Gtk.ScrolledWindow):
 
     def _selection_validation(self, selection, model, path, current):
         """
-            Keep track of last inserted item
+            Check if selection is valid
             @param selection as Gtk.TreeSelection
             @param model as Gtk.TreeModel
             @param path as Gtk.TreePath
@@ -371,7 +370,7 @@ class SelectionList(Gtk.ScrolledWindow):
             @return bool
         """
         ids = self.get_selected_ids()
-        if not ids:
+        if not ids or self._mode == SelectionMode.NORMAL:
             return True
         elif self._modifier:
             iterator = self._model.get_iter(path)
@@ -400,6 +399,7 @@ class SelectionList(Gtk.ScrolledWindow):
         self._timeout = None
 
     def _on_button_press_event(self, view, event):
+        view.grab_focus()
         state = event.get_state()
         if state & Gdk.ModifierType.CONTROL_MASK or\
            state & Gdk.ModifierType.SHIFT_MASK:
