@@ -21,7 +21,6 @@ from gettext import gettext as _
 from threading import Thread
 
 from lollypop.define import Lp, Type, SecretSchema, SecretAttributes, ArtSize
-from lollypop.mpd import MpdServerDaemon
 
 
 class Settings(Gio.Settings):
@@ -69,7 +68,8 @@ class SettingsDialog:
             Init dialog
         """
         self._choosers = []
-        self._timeout_id = None
+        self._cover_tid = None
+        self._mix_tid = None
         self._popover = None
 
         builder = Gtk.Builder()
@@ -82,9 +82,12 @@ class SettingsDialog:
         self._settings_dialog = builder.get_object('settings_dialog')
         self._settings_dialog.set_transient_for(Lp().window)
 
-        if not Lp().settings.get_value('disable-csd'):
-            self._settings_dialog.set_titlebar(
-                builder.get_object('header_bar'))
+        if Lp().settings.get_value('disable-csd'):
+            self._settings_dialog.set_title(_("Preferences"))
+        else:
+            headerbar = builder.get_object('header_bar')
+            headerbar.set_title(_("Preferences"))
+            self._settings_dialog.set_titlebar(headerbar)
 
         switch_scan = builder.get_object('switch_scan')
         switch_scan.set_state(Lp().settings.get_value('auto-update'))
@@ -98,12 +101,6 @@ class SettingsDialog:
         switch_state = builder.get_object('switch_state')
         switch_state.set_state(Lp().settings.get_value('save-state'))
 
-        switch_autoplay = builder.get_object('switch_autoplay')
-        switch_autoplay.set_state(Lp().settings.get_value('auto-play'))
-
-        switch_mpd = builder.get_object('switch_mpd')
-        switch_mpd.set_state(not Lp().settings.get_value('disable-mpd'))
-
         switch_mix = builder.get_object('switch_mix')
         switch_mix.set_state(Lp().settings.get_value('mix'))
 
@@ -117,8 +114,11 @@ class SettingsDialog:
         switch_compilations.set_state(
             Lp().settings.get_value('show-compilations'))
 
+        switch_repeat = builder.get_object('switch_repeat')
+        switch_repeat.set_state(not Lp().settings.get_value('repeat'))
+
         scale_coversize = builder.get_object('scale_coversize')
-        scale_coversize.set_range(100, 300)
+        scale_coversize.set_range(150, 300)
         scale_coversize.set_value(
                             Lp().settings.get_value('cover-size').get_int32())
         self._settings_dialog.connect('destroy', self._edit_settings_close)
@@ -229,19 +229,19 @@ class SettingsDialog:
             Delayed update cover size
             @param widget as Gtk.Range
         """
-        if self._timeout_id is not None:
-            GLib.source_remove(self._timeout_id)
-            self._timeout_id = None
-        self._timeout_id = GLib.timeout_add(500,
-                                            self._really_update_coversize,
-                                            widget)
+        if self._cover_tid is not None:
+            GLib.source_remove(self._cover_tid)
+            self._cover_tid = None
+        self._cover_tid = GLib.timeout_add(500,
+                                           self._really_update_coversize,
+                                           widget)
 
     def _really_update_coversize(self, widget):
         """
             Update cover size
             @param widget as Gtk.Range
         """
-        self._timeout_id = None
+        self._cover_tid = None
         value = widget.get_value()
         Lp().settings.set_value('cover-size', GLib.Variant('i', value))
         ArtSize.BIG = value
@@ -256,7 +256,6 @@ class SettingsDialog:
         if not Lp().player.is_party():
             settings = Gtk.Settings.get_default()
             settings.set_property("gtk-application-prefer-dark-theme", state)
-            Lp().window.update_view()
 
     def _update_scan_setting(self, widget, state):
         """
@@ -291,30 +290,6 @@ class SettingsDialog:
         Lp().settings.set_value('show-genres',
                                 GLib.Variant('b', state))
 
-    def _update_autoplay_setting(self, widget, state):
-        """
-            Update auto play setting
-            @param widget as unused, state as widget state
-        """
-        Lp().settings.set_value('auto-play',
-                                GLib.Variant('b', state))
-        Lp().window.update_view()
-
-    def _update_mpd_setting(self, widget, state):
-        """
-            Update mpd setting
-            @param widget as unused, state as widget state
-        """
-        Lp().settings.set_value('disable-mpd',
-                                GLib.Variant('b', not state))
-        if Lp().mpd is None:
-            Lp().mpd = MpdServerDaemon(
-                               Lp().settings.get_value('mpd-eth').get_string(),
-                               Lp().settings.get_value('mpd-port').get_int32())
-        else:
-            Lp().mpd.quit()
-            Lp().mpd = None
-
     def _update_mix_setting(self, widget, state):
         """
             Update mix setting
@@ -355,7 +330,14 @@ class SettingsDialog:
         """
         Lp().settings.set_value('show-compilations',
                                 GLib.Variant('b', state))
-        Lp().window.update_view()
+
+    def _update_repeat_setting(self, widget, state):
+        """
+            Update repeat setting
+            @param widget as unused, state as widget state
+        """
+        Lp().settings.set_value('repeat',
+                                GLib.Variant('b', not state))
 
     def _update_lastfm_settings(self, sync=False):
         """
@@ -417,6 +399,7 @@ class SettingsDialog:
         self._settings_dialog.destroy()
         if set(previous) != set(paths):
             Lp().window.update_db()
+        Lp().window.update_view()
 
     def _party_switch_state(self, widget, state, genre_id):
         """
@@ -436,12 +419,12 @@ class SettingsDialog:
                 pass
         Lp().settings.set_value('party-ids',  GLib.Variant('ai', ids))
 
-    def _on_mix_enter_notify(self, widget, event):
+    def _show_mix_popover(self, widget):
         """
             Show mix popover
             @param widget as Gtk.Widget
-            @param event as Gdk.Event
         """
+        self._mix_tid = None
         if Lp().settings.get_value('mix'):
             if self._popover is None:
                 self._popover = Gtk.Popover.new(widget)
@@ -449,6 +432,26 @@ class SettingsDialog:
                 self._popover.add(self._popover_content)
             self._popover.show_all()
 
+    def _on_mix_button_press(self, widget, event):
+        """
+            Show mix popover on long press
+            @param widget as Gtk.Widget
+            @param event as Gdk.Event
+        """
+        self._mix_tid = GLib.timeout_add(500, self._show_mix_popover, widget)
+
+    def _on_mix_button_release(self, widget, event):
+        """
+            If no popover shown, pass event
+            @param widget as Gtk.Widget
+            @param event as Gdk.Event
+        """
+        if self._mix_tid is None:
+            return True
+        else:
+            GLib.source_remove(self._mix_tid)
+            self._mix_tid = None
+
     def _on_key_release_event(self, widget, event):
         """
             Destroy window if Esc
