@@ -10,12 +10,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gio
+from gi.repository import Gtk, Gio, GLib
 
 from lollypop.pop_next import NextPopover
-from lollypop.pop_queue import QueueWidget
 from lollypop.pop_search import SearchPopover
-from lollypop.define import Lp, Shuffle
+from lollypop.pop_albums import AlbumsPopover
+from lollypop.pop_playlists import PlaylistsPopover
+from lollypop.pop_queue import QueuePopover
+from lollypop.pop_externals import ExternalsPopover
+from lollypop.define import Lp, Shuffle, Type
 
 
 class ToolbarEnd(Gtk.Bin):
@@ -31,9 +34,9 @@ class ToolbarEnd(Gtk.Bin):
         Gtk.Bin.__init__(self)
         self.connect('show', self._on_show)
         self.connect('hide', self._on_hide)
-        self._pop_next = NextPopover()
-        self._queue = None
+        self._next_popover = NextPopover()
         self._search = None
+        self._timeout_id = None
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Lollypop/ToolbarEnd.ui')
         builder.connect_signals(self)
@@ -42,37 +45,37 @@ class ToolbarEnd(Gtk.Bin):
 
         self._grid_next = builder.get_object('grid-next')
 
-        self._shuffle_btn = builder.get_object('shuffle-button')
-        self._shuffle_btn_image = builder.get_object('shuffle-button-image')
-        shuffleAction = Gio.SimpleAction.new('shuffle-btn', None)
+        self._shuffle_button = builder.get_object('shuffle-button')
+        self._shuffle_image = builder.get_object('shuffle-button-image')
+        shuffleAction = Gio.SimpleAction.new('shuffle-button', None)
         shuffleAction.connect('activate', self._activate_shuffle_button)
         app.add_action(shuffleAction)
-        app.set_accels_for_action("app.shuffle-btn", ["<Control>r"])
-        Lp().settings.connect('changed::shuffle', self._shuffle_btn_aspect)
+        app.set_accels_for_action("app.shuffle-button", ["<Control>r"])
+        Lp().settings.connect('changed::shuffle', self._on_shuffle_changed)
 
-        self._party_btn = builder.get_object('party-button')
+        self._party_button = builder.get_object('party-button')
         party_action = Gio.SimpleAction.new('party', None)
         party_action.connect('activate', self._activate_party_button)
         app.add_action(party_action)
         app.set_accels_for_action("app.party", ["<Control>p"])
 
         self._search_button = builder.get_object('search-button')
-        searchAction = Gio.SimpleAction.new('search', None)
-        searchAction.connect('activate', self._on_search_btn_clicked)
-        app.add_action(searchAction)
+        search_action = Gio.SimpleAction.new('search', None)
+        search_action.connect('activate', self._on_search_button_clicked)
+        app.add_action(search_action)
         app.set_accels_for_action("app.search", ["<Control>f"])
 
-        self._queue_button = builder.get_object('queue-button')
-        queueAction = Gio.SimpleAction.new('queue', None)
-        queueAction.connect('activate', self._on_queue_btn_clicked)
-        app.add_action(queueAction)
-        app.set_accels_for_action("app.queue", ["<Control>l"])
-
         self._settings_button = builder.get_object('settings-button')
 
+        self._list_button = builder.get_object('list-button')
+        list_action = Gio.SimpleAction.new('list', None)
+        list_action.connect('activate', self._on_list_button_clicked)
+        app.add_action(list_action)
+        app.set_accels_for_action("app.list", ["<Control>l"])
+        self._list_popover = None
         Lp().player.connect('party-changed', self._on_party_changed)
 
-    def setup_menu_btn(self, menu):
+    def setup_menu(self, menu):
         """
             Add an application menu to menu button
             @parma: menu as Gio.Menu
@@ -88,7 +91,7 @@ class ToolbarEnd(Gtk.Bin):
         if player.is_playing():
             # Party mode can be activated
             # via Fullscreen class, so check button state
-            self._party_btn.set_active(player.is_party())
+            self._party_button.set_active(player.is_party())
 
     def on_next_changed(self, player, force=False):
         """
@@ -96,31 +99,50 @@ class ToolbarEnd(Gtk.Bin):
             @param player as Player
             @param force to show the popover
         """
-        if not self.is_visible() or not self._pop_next.should_be_shown():
-            if not force:
-                self._pop_next.hide()
-                return
-        # Do not show next popover for non internal tracks as
-        # tags will be readed on the fly
-        if player.next_track.id is not None and\
-           player.next_track.id >= 0 and\
-            (player.is_party() or
-             Lp().settings.get_enum('shuffle') == Shuffle.TRACKS):
-            self._pop_next.update()
-            if not self._pop_next.is_visible():
-                self._pop_next.set_relative_to(self._grid_next)
-                self._pop_next.show()
+        # Do not show popover is this menu is active
+        # or if we are hidden
+        if self._shuffle_button.get_active() or\
+           not self._grid_next.is_visible():
+            return
+        self._timeout_id = None
+        if self._next_popover.should_be_shown() or force:
+            self._next_popover.update()
+            if not self._next_popover.is_visible():
+                self._next_popover.set_relative_to(self._grid_next)
+                self._next_popover.show()
         else:
-            self._pop_next.hide()
+            self._next_popover.hide()
 
 #######################
 # PRIVATE             #
 #######################
-    def _on_btn_enter_notify(self, button, event):
+    def _on_shuffle_button_clicked(self, button):
         """
-            Show next popover
+            Hide next popover
+            @param button as Gtk.Button
         """
-        self.on_next_changed(Lp().player, True)
+        self._next_popover.hide()
+
+    def _on_button_press(self, button, event):
+        """
+            Show next popover on long press
+            @param button as Gtk.Button
+            @param event as Gdk.Event
+        """
+        self._timeout_id = GLib.timeout_add(500, self.on_next_changed,
+                                            Lp().player, True)
+
+    def _on_button_release(self, button, event):
+        """
+            If next popover shown, block event
+            @param button as Gtk.Button
+            @param event as Gdk.Event
+        """
+        if self._timeout_id is None:
+            return True
+        else:
+            GLib.source_remove(self._timeout_id)
+            self._timeout_id = None
 
     def _set_shuffle_icon(self):
         """
@@ -128,29 +150,19 @@ class ToolbarEnd(Gtk.Bin):
         """
         shuffle = Lp().settings.get_enum('shuffle')
         if shuffle == Shuffle.NONE:
-            self._shuffle_btn_image.get_style_context().remove_class(
-                                                                    'selected')
-            self._shuffle_btn_image.set_from_icon_name(
+            self._shuffle_image.get_style_context().remove_class('selected')
+            self._shuffle_image.set_from_icon_name(
                 "media-playlist-consecutive-symbolic",
                 Gtk.IconSize.SMALL_TOOLBAR)
         else:
-            self._shuffle_btn_image.set_from_icon_name(
+            self._shuffle_image.set_from_icon_name(
                 "media-playlist-shuffle-symbolic",
                 Gtk.IconSize.SMALL_TOOLBAR)
             if shuffle == Shuffle.TRACKS:
-                self._shuffle_btn_image.get_style_context().add_class(
-                                                                    'selected')
+                self._shuffle_image.get_style_context().add_class('selected')
             else:
-                self._shuffle_btn_image.get_style_context().remove_class(
+                self._shuffle_image.get_style_context().remove_class(
                                                                     'selected')
-        self.on_next_changed(Lp().player)
-
-    def _shuffle_btn_aspect(self, settings, value):
-        """
-            Mark shuffle button as active when shuffle active
-            @param settings as Gio.Settings, value as str
-        """
-        self._set_shuffle_icon()
 
     def _activate_party_button(self, action=None, param=None):
         """
@@ -158,7 +170,7 @@ class ToolbarEnd(Gtk.Bin):
             @param action as Gio.SimpleAction
             @param param as GLib.Variant
         """
-        self._party_btn.set_active(not self._party_btn.get_active())
+        self._party_button.set_active(not self._party_button.get_active())
         Lp().window.responsive_design()
 
     def _activate_shuffle_button(self, action=None, param=None):
@@ -167,35 +179,35 @@ class ToolbarEnd(Gtk.Bin):
             @param action as Gio.SimpleAction
             @param param as GLib.Variant
         """
-        self._shuffle_btn.set_active(not self._shuffle_btn.get_active())
+        self._shuffle_button.set_active(not self._shuffle_button.get_active())
 
-    def _on_search_btn_clicked(self, obj, param=None):
+    def _on_shuffle_changed(self, settings, value):
+        """
+            Mark shuffle button as active when shuffle active
+            @param settings as Gio.Settings, value as str
+        """
+        self._set_shuffle_icon()
+        self._next_popover.hide()
+
+    def _on_search_button_clicked(self, obj, param=None):
         """
             Show search widget on search button clicked
             @param obj as Gtk.Button or Gtk.Action
         """
         if self._search is None:
-            self._search = SearchPopover(self)
+            self._search = SearchPopover()
+            self._search.connect('closed', self._on_popover_closed)
         self._search.set_relative_to(self._search_button)
+        self._next_popover.hide()
         self._search.show()
 
-    def _on_queue_btn_clicked(self, button, param=None):
-        """
-            Show queue widget on queue button clicked
-            @param obj as Gtk.Button or Gtk.Action
-        """
-        if self._queue is None:
-            self._queue = QueueWidget()
-        self._queue.set_relative_to(self._queue_button)
-        self._queue.show()
-
-    def _on_party_btn_toggled(self, button):
+    def _on_party_button_toggled(self, button):
         """
             Set party mode on if party button active
             @param obj as Gtk.button
         """
-        active = self._party_btn.get_active()
-        self._shuffle_btn.set_sensitive(not active)
+        active = self._party_button.get_active()
+        self._shuffle_button.set_sensitive(not active)
         if not Lp().settings.get_value('dark-ui'):
             settings = Gtk.Settings.get_default()
             settings.set_property("gtk-application-prefer-dark-theme", active)
@@ -208,9 +220,53 @@ class ToolbarEnd(Gtk.Bin):
             @param player as Player
             @param is party as bool
         """
-        if self._party_btn.get_active() != is_party:
+        if self._party_button.get_active() != is_party:
             self._activate_party_button()
 
+    def _on_list_button_clicked(self, widget, unused=None):
+        """
+            Show current playback context popover
+            @param widget as Gtk.Widget
+        """
+        if self._list_popover is not None:
+            return
+        self._next_popover.hide()
+        if Lp().player.current_track.id == Type.EXTERNALS:
+            self._list_popover = ExternalsPopover()
+            self._list_popover.set_relative_to(self._list_button)
+            self._list_popover.populate()
+            self._list_popover.show()
+        elif Lp().player.get_queue():
+            self._list_popover = QueuePopover()
+            self._list_popover.set_relative_to(self._list_button)
+            self._list_popover.show()
+        elif Lp().player.get_user_playlist_ids():
+            self._list_popover = PlaylistsPopover()
+            self._list_popover.set_relative_to(self._list_button)
+            self._list_popover.show()
+        else:
+            self._list_popover = AlbumsPopover()
+            self._list_popover.set_relative_to(self._list_button)
+            self._list_popover.show()
+        self._list_popover.connect('closed', self._on_list_popover_closed)
+        return True
+
+    def _on_list_popover_closed(self, popover):
+        """
+            Reset variable
+            @param popover as Gtk.Popover
+        """
+        self._list_popover = None
+        self._on_popover_closed(popover)
+
+    def _on_popover_closed(self, popover):
+        """
+            Restore next popover if needed
+            @param popover as Gtk.Popover
+        """
+        if self._next_popover.should_be_shown():
+            self._next_popover.show()
+
     def _on_show(self, widget):
         """
             Show popover if needed
@@ -223,4 +279,4 @@ class ToolbarEnd(Gtk.Bin):
             Hide popover
             @param widget as Gtk.Widget
         """
-        self._pop_next.hide()
+        self._next_popover.hide()
