@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, Gdk, GLib
 from cgi import escape
 
 from lollypop.pop_menu import TrackMenuPopover
@@ -38,18 +38,21 @@ class ToolbarInfos(Gtk.Bin, InfosController):
         builder.connect_signals(self)
         self._pop_tunein = None
         self._pop_infos = None
+        self._timeout_id = None
         self._width = 0
 
         self._infobox = builder.get_object('infos')
-        self._infobox.set_property('has-tooltip', True)
         self.add(self._infobox)
 
         self._labels = builder.get_object('nowplaying_labels')
+
         self._title_label = builder.get_object('title')
         self._artist_label = builder.get_object('artist')
         self._cover_frame = builder.get_object('frame')
         self._cover = builder.get_object('cover')
+        self._cover.set_property('has-tooltip', True)
 
+        self.connect('realize', self._on_realize)
         Lp().art.connect('album-artwork-changed', self._update_cover)
         Lp().art.connect('radio-artwork-changed', self._update_logo)
 
@@ -103,38 +106,59 @@ class ToolbarInfos(Gtk.Bin, InfosController):
             self._cover.set_from_surface(pixbuf)
             del pixbuf
 
-    def _on_infobox_clicked(self, eventbox, event):
+    def _show_track_menu(self):
         """
-            Pop albums from current artistleft click
-            Show playlist menu on right
-            @param eventbox as Gtk.EventBox
+            Show current track menu
+        """
+        self._timeout_id = None
+        if Lp().player.current_track.id >= 0:
+            popover = TrackMenuPopover(
+                        Lp().player.current_track.id,
+                        PopToolbarMenu(Lp().player.current_track.id))
+            popover.set_relative_to(self._infobox)
+            popover.show()
+
+    def _on_title_press_event(self, widget, event):
+        """
+            On long press: show current track menu
+            @param widget as Gtk.Widget
+            @param event as Gdk.Event
+
+        """
+        self._timeout_id = GLib.timeout_add(500, self._show_track_menu)
+        return True
+
+    def _on_title_release_event(self, widget, event):
+        """
+            Show track information popover
+            On long press/right click: show current track menu
+            @param widget as Gtk.Widget
             @param event as Gdk.Event
         """
-        if Lp().player.current_track.id == Type.EXTERNALS:
-            expopover = ExternalsPopover()
-            expopover.set_relative_to(eventbox)
-            expopover.populate()
-            expopover.show()
-        elif Lp().player.current_track.id is not None:
-            if event.button == 1:
-                if Lp().player.current_track.id == Type.RADIOS:
-                    if self._pop_tunein is None:
-                        self._pop_tunein = TuneinPopover()
-                        self._pop_tunein.populate()
-                        self._pop_tunein.set_relative_to(self._infobox)
-                    self._pop_tunein.show()
+        if self._timeout_id is not None:
+            GLib.source_remove(self._timeout_id)
+            self._timeout_id = None
+            if Lp().player.current_track.id == Type.EXTERNALS:
+                expopover = ExternalsPopover()
+                expopover.set_relative_to(widget)
+                expopover.populate()
+                expopover.show()
+            elif Lp().player.current_track.id is not None:
+                if event.button == 1:
+                    if Lp().player.current_track.id == Type.RADIOS:
+                        if self._pop_tunein is None:
+                            self._pop_tunein = TuneinPopover()
+                            self._pop_tunein.populate()
+                            self._pop_tunein.set_relative_to(widget)
+                        self._pop_tunein.show()
+                    else:
+                        if self._pop_infos is None:
+                            self._pop_infos = InfosPopover()
+                            self._pop_infos.set_relative_to(widget)
+                        self._pop_infos.show()
                 else:
-                    if self._pop_infos is None:
-                        self._pop_infos = InfosPopover()
-                        self._pop_infos.set_relative_to(self._infobox)
-                    self._pop_infos.show()
-            elif Lp().player.current_track.id >= 0:
-                popover = TrackMenuPopover(
-                            Lp().player.current_track.id,
-                            PopToolbarMenu(Lp().player.current_track.id))
-                popover.set_relative_to(self._infobox)
-                popover.show()
-            return True
+                    self._show_track_menu()
+        return True
 
     def _on_query_tooltip(self, widget, x, y, keyboard, tooltip):
         """
@@ -157,12 +181,17 @@ class ToolbarInfos(Gtk.Bin, InfosController):
             return False
         return True
 
-    def _on_eventbox_realize(self, eventbox):
+    def _on_realize(self, toolbar):
         """
-            Show hand cursor over
+            @param toolbar as ToolbarInfos
         """
         style = self.get_style_context()
         padding = style.get_padding(style.get_state())
         ArtSize.SMALL = self.get_allocated_height()\
             - padding.top - padding.bottom - 2
+
+    def _on_eventbox_realize(self, eventbox):
+        """
+            Show hand cursor over
+        """
         eventbox.get_window().set_cursor(Gdk.Cursor(Gdk.CursorType.HAND1))
