@@ -138,17 +138,6 @@ def seconds_to_string(duration):
     return '%i:%02i' % (minutes, seconds)
 
 
-def rgba_to_hex(color):
-    """
-        Convert Gdk.Color to hexadecimal
-        @param Gdk.Color
-        @return string
-    """
-    return "#{0:02x}{1:02x}{2:02x}".format(int(color.red * 255),
-                                           int(color.green * 255),
-                                           int(color.blue * 255))
-
-
 def is_loved(track_id):
     """
         Check if object is in loved playlist
