@@ -12,7 +12,7 @@
 
 from gi.repository import Gtk, GLib
 
-from lollypop.define import Lp
+from lollypop.define import Lp, ArtSize
 
 
 class View(Gtk.Grid):
@@ -32,14 +32,13 @@ class View(Gtk.Grid):
                                                    self._on_current_changed)
         self._cover_signal = Lp().art.connect('album-artwork-changed',
                                               self._on_cover_changed)
-        self._scan_signal = Lp().scanner.connect('album-modified',
-                                                 self._on_album_modified)
 
         # Stop populate thread
         self._stop = False
         self._new_ids = []
 
         self._scrolled = Gtk.ScrolledWindow()
+        self._scrolled.connect('leave-notify-event', self._on_leave_event)
         self._scrolled.show()
         self._viewport = Gtk.Viewport()
         self._scrolled.add(self._viewport)
@@ -59,12 +58,11 @@ class View(Gtk.Grid):
         """
         GLib.idle_add(self._update_widgets, self._get_children())
 
-    def update_responsive_design(self):
+    def update_overlays(self):
         """
-            Update the view based on current size
+            Update overlays
         """
-        GLib.idle_add(self._update_responsive_design,
-                      self._get_children())
+        GLib.idle_add(self._update_overlays, self._get_children())
 
     def populate(self):
         pass
@@ -72,6 +70,27 @@ class View(Gtk.Grid):
 #######################
 # PRIVATE             #
 #######################
+    def _on_leave_event(self, widget, event):
+        """
+            Update children's overlay
+        """
+        allocation = widget.get_allocation()
+        if event.x < 5 or\
+           event.x > allocation.width - 5 or\
+           event.y < 5 or\
+           event.y > allocation.height - 5:
+            self._update_overlays(self._get_children())
+
+    def _update_overlays(self, widgets):
+        """
+            Update children's overlay
+            @param widgets as AlbumWidget
+        """
+        if widgets:
+            widget = widgets.pop(0)
+            widget.update_overlay()
+            GLib.idle_add(self._update_overlays, widgets)
+
     def _update_widgets(self, widgets):
         """
             Update all widgets
@@ -80,19 +99,9 @@ class View(Gtk.Grid):
         if widgets:
             widget = widgets.pop(0)
             widget.update_state()
-            widget.update_cursor()
             widget.update_playing_indicator()
             GLib.idle_add(self._update_widgets, widgets)
 
-    def _update_responsive_design(self, widgets):
-        """
-            Update widgets
-        """
-        if widgets:
-            widget = widgets.pop(0)
-            widget.responsive_design()
-            GLib.idle_add(self._update_responsive_design, widgets)
-
     def _get_children(self):
         """
             Return view children
@@ -110,9 +119,6 @@ class View(Gtk.Grid):
         if self._cover_signal:
             Lp().art.disconnect(self._cover_signal)
             self._cover_signal = None
-        if self._scan_signal:
-            Lp().scanner.disconnect(self._scan_signal)
-            self._scan_signal = None
 
     def _on_cover_changed(self, art, album_id):
         """
@@ -130,12 +136,91 @@ class View(Gtk.Grid):
         """
         GLib.idle_add(self._update_widgets, self._get_children())
 
-    def _on_album_modified(self, scanner, album_id):
+
+class LazyLoadingView(View):
+    """
+        Lazy loading for view
+    """
+
+    def __init__(self):
         """
-            On album modified, disable it
-            @param scanner as CollectionScanner
-            @param album id as int
+            Init lazy loading
         """
-        for child in self._get_children():
-            if album_id == child.get_id():
-                child.set_sensitive(False)
+        View.__init__(self)
+        self._lazy_queue = []  # Widgets not initialized
+        self._scroll_value = 0
+        self._timeout_id = None
+        self._scrolled.get_vadjustment().connect('value-changed',
+                                                 self._on_value_changed)
+
+    def append(self, row):
+        """
+            Append row to lazy queue
+            @param row as Row
+        """
+        self._lazy_queue.append(row)
+
+    def lazy_loading(self, widgets=[], scroll_value=0):
+        """
+            Load the view in a lazy way:
+                - widgets first
+                - _waiting_init then
+            @param widgets as [AlbumSimpleWidgets]
+            @param scroll_value as float
+        """
+        widget = None
+        if self._stop or self._scroll_value != scroll_value:
+            return
+        if widgets:
+            widget = widgets.pop(0)
+            self._lazy_queue.remove(widget)
+        elif self._lazy_queue:
+            widget = self._lazy_queue.pop(0)
+        if widget is not None:
+            widget.populate()
+            if self._timeout_id is None:
+                GLib.idle_add(self.lazy_loading, widgets, scroll_value)
+            else:
+                GLib.timeout_add(50, self.lazy_loading, widgets, scroll_value)
+
+#######################
+# PRIVATE             #
+#######################
+    def _is_visible(self, widget):
+        """
+            Is widget visible in scrolled
+            @param widget as Gtk.Widget
+        """
+        widget_alloc = widget.get_allocation()
+        scrolled_alloc = self._scrolled.get_allocation()
+        try:
+            (x, y) = widget.translate_coordinates(self._scrolled, 0, 0)
+            return (y > -widget_alloc.height-ArtSize.BIG or y >= 0) and\
+                y < scrolled_alloc.height+ArtSize.BIG
+        except:
+            return True
+
+    def _lazy_or_not(self, adj):
+        """
+            Add visible widgets to lazy queue
+            @param adj as Gtk.Adjustment
+        """
+        self._timeout_id = None
+        self._scroll_value = adj.get_value()
+        widgets = []
+        for child in self._lazy_queue:
+            if self._is_visible(child):
+                widgets.append(child)
+        GLib.idle_add(self.lazy_loading, widgets, self._scroll_value)
+
+    def _on_value_changed(self, adj):
+        """
+            Update scroll value and check for lazy queue
+            @param adj as Gtk.Adjustment
+        """
+        if self._timeout_id is not None:
+            GLib.source_remove(self._timeout_id)
+            self._timeout_id = None
+        if not self._lazy_queue:
+            return
+        self._timeout_id = GLib.timeout_add(250, self._lazy_or_not, adj)
