@@ -12,6 +12,8 @@
 
 from gi.repository import Gtk, Gdk
 
+from cgi import escape
+
 from lollypop.define import Lp
 from lollypop.pop_infos import InfosPopover
 from lollypop.view_artist_albums import ArtistAlbumsView
@@ -29,15 +31,17 @@ class ArtistView(ArtistAlbumsView):
             @param genre id as int
         """
         ArtistAlbumsView.__init__(self, artist_ids, genre_ids)
+        self._signal_id = None
         if len(artist_ids) > 1:
             self._artist_id = None
         else:
             self._artist_id = artist_ids[0]
-        self._signal_id = None
 
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Lollypop/ArtistView.ui')
         builder.connect_signals(self)
+        self._jump_button = builder.get_object('jump-button')
+        self._spinner = builder.get_object('spinner')
         self.attach(builder.get_object('ArtistView'), 0, 0, 1, 1)
         artists = ""
         for artist_id in artist_ids:
@@ -47,6 +51,52 @@ class ArtistView(ArtistAlbumsView):
 #######################
 # PRIVATE             #
 #######################
+    def _update_jump_button(self):
+        """
+            Update jump button status
+        """
+        found = False
+        for child in self._get_children():
+            if child.get_id() == Lp().player.current_track.album.id:
+                found = True
+                break
+        if found:
+            self._jump_button.set_sensitive(True)
+            self._jump_button.set_tooltip_markup(
+                                      "<b>%s</b> - " %
+                                      escape(
+                                        Lp().player.current_track.artist_names
+                                        ) +
+                                      Lp().player.current_track.name)
+        else:
+            self._jump_button.set_sensitive(False)
+            self._jump_button.set_tooltip_text('')
+
+    def _on_jump_button_clicked(self, widget):
+        """
+            Scroll to album
+        """
+        self.jump_to_current()
+
+    def _on_populated(self, widget, widgets, scroll_value):
+        """
+            Set jump button state
+            @param widget as AlbumDetailedWidget
+            @param widgets as pending AlbumDetailedWidgets
+            @param scroll value as float
+        """
+        self._update_jump_button()
+        self._spinner.stop()
+        ArtistAlbumsView._on_populated(self, widget, widgets, scroll_value)
+
+    def _on_current_changed(self, player):
+        """
+            Set playing button status
+            @param player as Player
+        """
+        ArtistAlbumsView._on_current_changed(self, player)
+        self._update_jump_button()
+
     def _on_label_realize(self, eventbox):
         """
             Change pointer on label
