@@ -10,20 +10,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk, GLib, GObject
 
-from threading import Thread
-
-from lollypop.view import View
+from lollypop.view import LazyLoadingView, View
 from lollypop.view_container import ViewContainer
 from lollypop.define import Lp, Type
 from lollypop.widgets_album import AlbumDetailedWidget
 
 
-class ArtistAlbumsView(View):
+class ArtistAlbumsView(LazyLoadingView):
     """
         Show artist albums and tracks
     """
+    __gsignals__ = {
+        'populated': (GObject.SignalFlags.RUN_FIRST, None, ()),
+    }
 
     def __init__(self, artist_ids, genre_ids):
         """
@@ -31,9 +32,10 @@ class ArtistAlbumsView(View):
             @param artist ids as [int]
             @param genre ids as [int]
         """
-        View.__init__(self)
+        LazyLoadingView.__init__(self)
         self._artist_ids = artist_ids
         self._genre_ids = genre_ids
+        self._albums_count = 0
 
         self._albumbox = Gtk.Grid()
         self._albumbox.set_row_spacing(20)
@@ -42,19 +44,61 @@ class ArtistAlbumsView(View):
 
         self._scrolled.set_property('expand', True)
         self._viewport.set_property("valign", Gtk.Align.START)
-        self._viewport.add(self._albumbox)
         self.attach(self._scrolled, 0, 1, 1, 1)
 
     def populate(self, albums):
         """
             Populate the view
+            @param albums as [int]
         """
         if albums:
+            self._albums_count = len(albums)
             self._add_albums(albums)
 
+    def jump_to_current(self):
+        """
+            Jump to current album
+        """
+        widget = None
+        for child in self._albumbox.get_children():
+            if child.get_id() == Lp().player.current_track.album.id:
+                widget = child
+                break
+        if widget is not None:
+            y = widget.get_current_ordinate(self._albumbox)
+            self._scrolled.get_vadjustment().set_value(y)
+
+    def stop(self):
+        """
+            Stop loading
+        """
+        self._lazy_queue = []
+        LazyLoadingView.stop(self)
+
 #######################
 # PRIVATE             #
 #######################
+    def lazy_loading(self, widgets=[], scroll_value=0):
+        """
+            Load the view in a lazy way:
+                - widgets first
+                - _waiting_init then
+            @param widgets as [AlbumSimpleWidgets]
+            @param scroll_value as float
+        """
+        widget = None
+        if self._stop or self._scroll_value != scroll_value:
+            return
+        if widgets:
+            widget = widgets.pop(0)
+            self._lazy_queue.remove(widget)
+        elif self._lazy_queue:
+            widget = self._lazy_queue.pop(0)
+        if widget is not None:
+            widget.connect('populated', self._on_populated,
+                           widgets, scroll_value)
+            widget.populate()
+
     def _get_children(self):
         """
             Return view children
@@ -68,26 +112,35 @@ class ArtistAlbumsView(View):
             repeat operation until album list is empty
             @param [album ids as int]
         """
-        size_group = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
-        widget = AlbumDetailedWidget(albums.pop(0),
-                                     self._genre_ids,
-                                     self._artist_ids,
-                                     size_group)
-        widget.connect('finished', self._on_album_finished, albums)
-        widget.show()
-        t = Thread(target=widget.populate)
-        t.daemon = True
-        t.start()
-        self._albumbox.add(widget)
-
-    def _on_album_finished(self, album, albums):
-        """
-            Add another album
-            @param album as AlbumDetailedWidget
-            @param [album ids as int]
-        """
         if albums and not self._stop:
-            self._add_albums(albums)
+            album_id = albums.pop(0)
+            widget = AlbumDetailedWidget(album_id,
+                                         self._genre_ids,
+                                         self._artist_ids)
+            self._lazy_queue.append(widget)
+            # Not needed if only one album
+            if self._albums_count == 1:
+                widget.disable_play_all()
+            widget.show()
+            self._albumbox.add(widget)
+            GLib.idle_add(self._add_albums, albums)
+        else:
+            if self._viewport.get_child() is None:
+                self._viewport.add(self._albumbox)
+            self.emit('populated')
+            GLib.idle_add(self.lazy_loading)
+
+    def _on_populated(self, widget, widgets, scroll_value):
+        """
+            Add another album/disc
+            @param widget as AlbumDetailedWidget
+            @param widgets as pending AlbumDetailedWidgets
+            @param scroll value as float
+        """
+        if not widget.is_populated():
+            widget.populate()
+        elif not self._stop:
+            GLib.idle_add(self.lazy_loading, widgets, scroll_value)
         else:
             self._stop = False
 
@@ -119,6 +172,12 @@ class CurrentArtistAlbumsView(ViewContainer):
                 albums = [album_id]
             GLib.idle_add(self._populate, albums)
 
+    def jump_to_current(self):
+        """
+            Jump to current track
+        """
+        self.get_visible_child().jump_to_current()
+
 #######################
 # PRIVATE             #
 #######################
@@ -134,13 +193,24 @@ class CurrentArtistAlbumsView(ViewContainer):
             Populate view and make it visible
             @param albums as [albums ids as int]
         """
-        view = ArtistAlbumsView([], [])
-        view.show()
-        view.populate(albums)
+        # Add a loading indicator
+        view = View()
+        spinner = Gtk.Spinner()
+        spinner.set_hexpand(True)
+        spinner.set_property('halign', Gtk.Align.END)
+        spinner.start()
+        view.add(spinner)
+        view.show_all()
         self.add(view)
         self.set_visible_child(view)
         self.clean_old_views(view)
 
+        # Populate artist albums view
+        view = ArtistAlbumsView([self._current[0]], [])
+        view.connect('populated', self._on_populated, spinner)
+        view.show()
+        view.populate(albums)
+
     def _get_albums(self, artist_id):
         """
             Get albums
@@ -152,8 +222,13 @@ class CurrentArtistAlbumsView(ViewContainer):
             albums = Lp().artists.get_albums(artist_id)
         return albums
 
-    def do_get_preferred_width(self):
+    def _on_populated(self, view, spinner):
         """
-            Set 0 to force popover to not expand
+            Show the view
+            @param view as View
+            @param spinner as Gtk.Spinner
         """
-        return (0, 0)
+        spinner.stop()
+        self.add(view)
+        self.set_visible_child(view)
+        self.clean_old_views(view)
