@@ -12,47 +12,67 @@
 
 from gi.repository import Gtk
 
-from threading import Thread
+from gettext import gettext as _
+from cgi import escape
 
 from lollypop.view import View
-from lollypop.widgets_playlist import PlaylistWidget, PlaylistEditWidget
+from lollypop.widgets_playlist import PlaylistsWidget, PlaylistEditWidget
 from lollypop.widgets_playlist import PlaylistsManagerWidget
 from lollypop.define import Lp, Type
 
 
-class PlaylistView(View):
+class PlaylistsView(View):
     """
         Show playlist tracks
     """
 
-    def __init__(self, playlist_id):
+    def __init__(self, playlist_ids, editable=True):
         """
             Init PlaylistView
-            @parma playlist id as int
+            @parma playlist ids as [int]
+            @param editable as bool
         """
         View.__init__(self)
-        self._playlist_id = playlist_id
-        self._signal_id = Lp().playlists.connect('playlist-changed',
-                                                 self._update)
+        self._tracks = []
+        self._playlist_ids = playlist_ids
+        self._signal_id1 = Lp().playlists.connect('playlist-add',
+                                                  self._on_playlist_add)
+        self._signal_id2 = Lp().playlists.connect('playlist-del',
+                                                  self._on_playlist_del)
 
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Lollypop/PlaylistView.ui')
-        builder.get_object('title').set_label(
-                                          Lp().playlists.get_name(playlist_id))
         builder.connect_signals(self)
 
-        self._edit_btn = builder.get_object('edit_btn')
+        name = ""
+        for playlist_id in playlist_ids:
+            if playlist_id == Type.POPULARS:
+                name += _("Popular tracks")+", "
+            elif playlist_id == Type.RECENTS:
+                name += _("Recently played")+", "
+            elif playlist_id == Type.NEVER:
+                name += _("Never played")+", "
+            elif playlist_id == Type.RANDOMS:
+                name += _("Random tracks")+", "
+            elif playlist_id == Type.SEARCH:
+                name += _("Search")+", "
+            else:
+                name += Lp().playlists.get_name(playlist_id)+", "
+        builder.get_object('title').set_label(name[:-2])
 
-        if playlist_id < 0 and playlist_id != Type.LOVED:
-            self._edit_btn.hide()
-        self._back_btn = builder.get_object('back_btn')
+        self._edit_button = builder.get_object('edit-button')
+        self._jump_button = builder.get_object('jump-button')
+
+        if len(playlist_ids) > 1 or (
+           playlist_ids[0] < 0 and playlist_ids[0] != Type.LOVED) or\
+                not editable:
+            self._edit_button.hide()
         self._title = builder.get_object('title')
 
-        self._playlist_widget = PlaylistWidget(playlist_id)
-        self._playlist_widget.show()
-
+        self._playlists_widget = PlaylistsWidget(playlist_ids)
+        self._playlists_widget.show()
         self.add(builder.get_object('widget'))
-        self._viewport.add(self._playlist_widget)
+        self._viewport.add(self._playlists_widget)
         self._scrolled.set_property('expand', True)
         self.add(self._scrolled)
 
@@ -61,24 +81,26 @@ class PlaylistView(View):
             Populate view with tracks from playlist
             Thread safe
         """
+        self._tracks = tracks
+        self._update_jump_button()
         mid_tracks = int(0.5+len(tracks)/2)
-        self._playlist_widget.populate_list_left(tracks[:mid_tracks],
-                                                 1)
-        self._playlist_widget.populate_list_right(tracks[mid_tracks:],
-                                                  mid_tracks + 1)
+        self._playlists_widget.populate_list_left(tracks[:mid_tracks],
+                                                  1)
+        self._playlists_widget.populate_list_right(tracks[mid_tracks:],
+                                                   mid_tracks + 1)
 
-    def get_id(self):
+    def get_ids(self):
         """
-            Return playlist id
-            @return id as int
+            Return playlist ids
+            @return id as [int]
         """
-        return self._playlist_id
+        return self._playlist_ids
 
     def stop(self):
         """
             Stop populating
         """
-        self._playlist_widget.stop()
+        self._playlists_widget.stop()
 
 #######################
 # PRIVATE             #
@@ -87,50 +109,84 @@ class PlaylistView(View):
         """
             Return view children
         """
-        return [self._playlist_widget]
+        return [self._playlists_widget]
 
-    def _update(self, manager, playlist_id):
+    def _update_jump_button(self):
+        """
+            Update jump button status
+        """
+        if Lp().player.current_track.id in self._tracks:
+            self._jump_button.set_sensitive(True)
+            self._jump_button.set_tooltip_markup(
+             "<b>%s</b>\n%s" % (escape(Lp().player.current_track.artist_names),
+                                escape(Lp().player.current_track.name)))
+        else:
+            self._jump_button.set_sensitive(False)
+            self._jump_button.set_tooltip_text('')
+
+    def _on_playlist_add(self, manager, playlist_id, track_id):
         """
             Update tracks widgets
             @param manager as PlaylistsManager
             @param playlist id as int
+            @param track id as int
         """
-        if playlist_id == self._playlist_id:
-            self._playlist_widget.clear()
-            t = Thread(target=self._update_view)
-            t.daemon = True
-            t.start()
+        if playlist_id in self._playlist_ids:
+            self._playlists_widget.append(track_id)
 
-    def _update_view(self):
+    def _on_playlist_del(self, manager, playlist_id, track_id):
         """
             Update tracks widgets
+            @param manager as PlaylistsManager
+            @param playlist id as int
+            @param track id as int
         """
-        tracks = Lp().playlists.get_tracks_ids(self._playlist_id)
-        self.populate(tracks)
+        if playlist_id in self._playlist_ids:
+            self._playlists_widget.remove(track_id)
+
+    def _on_populated(self, widget):
+        """
+            Show current track
+            @param widget as LazyLoadingView
+        """
+        self._update_jump_button()
 
     def _on_destroy(self, widget):
         """
             Disconnect signals
             @param widget as Gtk.Widget
         """
-        if self._signal_id:
-            Lp().playlists.disconnect(self._signal_id)
-            self._signal_id = None
+        View._on_destroy(self, widget)
+        if self._signal_id1:
+            Lp().playlists.disconnect(self._signal_id1)
+            self._signal_id1 = None
+        if self._signal_id2:
+            Lp().playlists.disconnect(self._signal_id2)
+            self._signal_id2 = None
 
-    def _on_edit_btn_clicked(self, button):
+    def _on_jump_button_clicked(self, button):
+        """
+            Scroll to current track
+            @param button as Gtk.Button
+        """
+        y = self._playlists_widget.get_current_ordinate()
+        if y is not None:
+            self._scrolled.get_vadjustment().set_value(y)
+
+    def _on_edit_button_clicked(self, button):
         """
             Edit playlist
             @param button as Gtk.Button
-            @param playlist name as str
         """
-        Lp().window.show_playlist_editor(self._playlist_id)
+        Lp().window.show_playlist_editor(self._playlist_ids[0])
 
     def _on_current_changed(self, player):
         """
-            Current song changed
+            Current song changed, update playing button
             @param player as Player
         """
-        self._playlist_widget.update_playing_indicator()
+        View._on_current_changed(self, player)
+        self._update_jump_button()
 
 
 class PlaylistsManageView(View):
@@ -224,4 +280,4 @@ class PlaylistEditView(View):
             Restore previous view
             @param button as Gtk.Button
         """
-        Lp().window.destroy_current_view()
+        Lp().window.reload_view()
