@@ -14,16 +14,15 @@ from gi.repository import Gtk, GLib
 
 from threading import Thread
 
-from lollypop.view import View
+from lollypop.view import LazyLoadingView
 from lollypop.widgets_radio import RadioWidget
 from lollypop.radios import Radios
 from lollypop.pop_radio import RadioPopover
 from lollypop.pop_tunein import TuneinPopover
 from lollypop.define import Lp, Type
-from lollypop.objects import Track
 
 
-class RadiosView(View):
+class RadiosView(LazyLoadingView):
     """
         Show radios in a grid
     """
@@ -32,7 +31,7 @@ class RadiosView(View):
         """
             Init view
         """
-        View.__init__(self)
+        LazyLoadingView.__init__(self)
         self._signal = Lp().art.connect('radio-artwork-changed',
                                         self._on_logo_changed)
 
@@ -48,11 +47,8 @@ class RadiosView(View):
         self._pop_tunein = TuneinPopover(self._radios_manager)
         self._pop_tunein.set_relative_to(builder.get_object('search_btn'))
 
-        self._sizegroup = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.BOTH)
-
         self._radiobox = Gtk.FlowBox()
         self._radiobox.set_selection_mode(Gtk.SelectionMode.NONE)
-        self._radiobox.connect("child-activated", self._on_album_activated)
         self._radiobox.set_property('column-spacing', 5)
         self._radiobox.set_property('row-spacing', 5)
         self._radiobox.set_homogeneous(True)
@@ -68,7 +64,6 @@ class RadiosView(View):
 
         self._viewport.set_property("valign", Gtk.Align.START)
         self._viewport.set_property('margin', 5)
-        self._viewport.add(self._radiobox)
         self._scrolled.set_property('expand', True)
 
         self.add(widget)
@@ -227,7 +222,7 @@ class RadiosView(View):
             widget = RadioWidget(radio,
                                  self._radios_manager)
             widget.show()
-            self._sizegroup.add_widget(widget)
+            self._lazy_queue.append(widget)
             if first:
                 self._radiobox.insert(widget, 0)
             else:
@@ -235,17 +230,6 @@ class RadiosView(View):
             GLib.idle_add(self._add_radios, radios)
         else:
             self._stop = False
-        return None
-
-    def _on_album_activated(self, flowbox, child):
-        """
-            Play album
-            @param flowbox as Gtk.Flowbox
-            @child as Gtk.FlowboxChild
-        """
-        name = child.get_child().get_name()
-        url = self._radios_manager.get_url(name)
-        if url:
-            track = Track()
-            track.set_radio(name, url)
-            Lp().player.load(track)
+            GLib.idle_add(self.lazy_loading)
+            if self._viewport.get_child() is None:
+                self._viewport.add(self._radiobox)
