@@ -18,51 +18,47 @@ from gettext import gettext as _
 
 from lollypop.define import Lp, Type, WindowSize
 from lollypop.cellrendereralbum import CellRendererAlbum
-from lollypop.widgets_track import TracksWidget
+from lollypop.widgets_track import TracksWidget, PlaylistRow
 from lollypop.objects import Track
 
 
-class PlaylistWidget(Gtk.Bin):
+class PlaylistsWidget(Gtk.Bin):
     """
         Show playlist tracks/albums
     """
 
-    def __init__(self, playlist_id):
+    def __init__(self, playlist_ids):
         """
             Init playlist Widget
-            @param playlist id as int
-            @param playlist name as str
+            @param playlist ids as [int]
         """
         Gtk.Bin.__init__(self)
-        self._playlist_id = playlist_id
+        self._playlist_ids = playlist_ids
         self._tracks1 = []
         self._tracks2 = []
+        self._width = None
+        self._orientation = None
         self._stop = False
+        # Used to block widget2 populate while showing one column
+        self._locked_widget2 = True
 
-        self._box = Gtk.FlowBox()
-        self._box.set_selection_mode(Gtk.SelectionMode.NONE)
-        self._box.set_hexpand(True)
+        self._box = Gtk.Grid()
+        self._box.set_column_homogeneous(True)
         self._box.set_property('valign', Gtk.Align.START)
         self._box.show()
 
-        loved = playlist_id != Type.LOVED
-        self._tracks_widget1 = TracksWidget(loved)
-        self._tracks_widget2 = TracksWidget(loved)
-        self._tracks_widget1.connect('activated',
-                                     self._on_activated)
-        self._tracks_widget2.connect('activated',
-                                     self._on_activated)
-        self._tracks_widget1.show()
-        self._tracks_widget2.show()
+        self.connect('size-allocate', self._on_size_allocate)
 
-        size_group = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
-        size_group.add_widget(self._tracks_widget1)
-        size_group.add_widget(self._tracks_widget2)
+        self._tracks_widget_left = TracksWidget()
+        self._tracks_widget_right = TracksWidget()
+        self._tracks_widget_left.connect('activated',
+                                         self._on_activated)
+        self._tracks_widget_right.connect('activated',
+                                          self._on_activated)
+        self._tracks_widget_left.show()
+        self._tracks_widget_right.show()
 
-        self._box.add(self._tracks_widget1)
-        self._box.add(self._tracks_widget2)
         self.add(self._box)
-        self.responsive_design()
 
     def get_id(self):
         """
@@ -71,50 +67,77 @@ class PlaylistWidget(Gtk.Bin):
         """
         return Type.PLAYLISTS
 
-    def responsive_design(self):
+    def update_overlay(self):
         """
-            Update the view based on current size
+            No overlay here now
         """
-        width = Lp().window.get_view_width()
-        if width < WindowSize.MONSTER:
-            self._box.set_min_children_per_line(1)
-            self._box.set_max_children_per_line(1)
-        else:
-            self._box.set_min_children_per_line(2)
-            self._box.set_max_children_per_line(2)
+        pass
+
+    def update_state(self):
+        """
+            No state to update
+        """
+        pass
+
+    def update_cover(self):
+        """
+            No update cover for now
+        """
+        pass
+
+    def get_current_ordinate(self):
+        """
+            If current track in widget, return it ordinate,
+            @return y as int
+        """
+        ordinate = None
+        for child in self._tracks_widget_left.get_children() + \
+                self._tracks_widget_right.get_children():
+            if child.get_id() == Lp().player.current_track.id:
+                ordinate = child.translate_coordinates(self._box, 0, 0)[1]
+        return ordinate
 
     def populate_list_left(self, tracks, pos):
         """
             Populate left list
-            @param track's ids as array of int
+            @param track's ids as array of int (not null)
             @param track position as int
             @thread safe
         """
-        self._stop = False
+        # We reset width here to allow size allocation code to run
+        self._width = None
+        self._tracks1 = list(tracks)
         GLib.idle_add(self._add_tracks,
                       tracks,
-                      self._tracks_widget1,
+                      self._tracks_widget_left,
                       pos)
 
     def populate_list_right(self, tracks, pos):
         """
             Populate right list
-            @param track's ids as array of int
+            @param track's ids as array of int (not null)
             @param track position as int
             @thread safe
         """
-        self._stop = False
-        GLib.idle_add(self._add_tracks,
-                      tracks,
-                      self._tracks_widget2,
-                      pos)
+        self._tracks2 = list(tracks)
+        # If we are showing only one column, wait for widget1
+        if self._orientation == Gtk.Orientation.VERTICAL and\
+           self._locked_widget2:
+            GLib.timeout_add(100, self.populate_list_right, tracks, pos)
+        else:
+            # We reset width here to allow size allocation code to run
+            self._width = None
+            GLib.idle_add(self._add_tracks,
+                          tracks,
+                          self._tracks_widget_right,
+                          pos)
 
     def update_playing_indicator(self):
         """
             Update playing indicator
         """
-        self._tracks_widget1.update_playing(Lp().player.current_track.id)
-        self._tracks_widget2.update_playing(Lp().player.current_track.id)
+        self._tracks_widget_left.update_playing(Lp().player.current_track.id)
+        self._tracks_widget_right.update_playing(Lp().player.current_track.id)
 
     def stop(self):
         """
@@ -122,15 +145,42 @@ class PlaylistWidget(Gtk.Bin):
         """
         self._stop = True
 
-    def clear(self):
+    def append(self, track_id):
         """
-            Clear tracks
+            Add track to widget
+            @param track id as int
         """
-        self._tracks = []
-        for child in self._tracks_widget1.get_children():
-            child.destroy()
-        for child in self._tracks_widget2.get_children():
-            child.destroy()
+        self._add_tracks([track_id],
+                         self._tracks_widget_right,
+                         -1)
+        self._update_tracks()
+        self._update_position()
+        self._update_headers()
+        self._tracks_widget_left.update_indexes(1)
+        self._tracks_widget_right.update_indexes(len(self._tracks1) + 1)
+
+    def remove(self, track_id):
+        """
+            Del track from widget
+            @param track id as int
+        """
+        children = self._tracks_widget_left.get_children() + \
+            self._tracks_widget_right.get_children()
+        # Clear the widget
+        if track_id is None:
+            for child in children:
+                child.destroy()
+            self._update_tracks()
+        else:
+            for child in children:
+                if child.get_id() == track_id:
+                    child.destroy()
+                    break
+            self._update_tracks()
+            self._update_position()
+            self._update_headers()
+            self._tracks_widget_left.update_indexes(1)
+            self._tracks_widget_right.update_indexes(len(self._tracks1) + 1)
 
 #######################
 # PRIVATE             #
@@ -141,37 +191,188 @@ class PlaylistWidget(Gtk.Bin):
             @param tracks id as array of [int]
             @param widget TracksWidget
             @param track position as int
+            @param pos as int
             @param previous album id as int
         """
         if not tracks or self._stop:
+            if widget == self._tracks_widget_right:
+                self._stop = False
+            else:
+                self._locked_widget2 = False
             return
 
         track = Track(tracks.pop(0))
-        name = escape(track.name)
-        album = track.album
+        row = PlaylistRow(track.id, pos,
+                          track.album.id != previous_album_id)
+        row.connect('track-moved', self._on_track_moved)
+        row.show()
+        widget.insert(row, pos)
+        GLib.idle_add(self._add_tracks, tracks, widget,
+                      pos + 1, track.album.id)
 
-        if track.id is None:
-            GLib.idle_add(self._add_tracks, tracks,
-                          widget, pos + 1, previous_album_id)
+    def _update_tracks(self):
+        """
+            Update tracks based on current widget
+        """
+        # Recalculate tracks
+        self._tracks1 = []
+        self._tracks2 = []
+        for child in self._tracks_widget_left.get_children():
+            self._tracks1.append(child.get_id())
+        for child in self._tracks_widget_right.get_children():
+            self._tracks2.append(child.get_id())
+
+    def _update_position(self):
+        """
+            Update widget position
+        """
+        len_tracks1 = len(self._tracks1)
+        len_tracks2 = len(self._tracks2)
+        # Take first track from tracks2 and put it at the end of tracks1
+        if len_tracks2 > len_tracks1:
+            src = self._tracks2[0]
+            if self._tracks1:
+                dst = self._tracks1[-1]
+            else:
+                dst = -1
+            self._move_track(dst, src, False)
+        # Take last track of tracks1 and put it at the bottom of tracks2
+        elif len_tracks1 - 1 > len_tracks2:
+            src = self._tracks1[-1]
+            if self._tracks2:
+                dst = self._tracks2[0]
+            else:
+                dst = -1
+            self._move_track(dst, src, True)
+        self._update_tracks()
+
+    def _update_headers(self):
+        """
+            Update headers for all tracks
+        """
+        self._tracks_widget_left.update_headers()
+        prev_album_id = None
+        if self._orientation == Gtk.Orientation.VERTICAL:
+            if self._tracks1:
+                prev_album_id = Track(self._tracks1[-1]).album.id
+        self._tracks_widget_right.update_headers(prev_album_id)
+
+    def _move_track(self, dst, src, up):
+        """
+            Move track from src to row
+            @param dst as int
+            @param src as int
+            @param up as bool
+            @return (dst_widget as TracksWidget,
+                     src index as int, dst index as int)
+        """
+        tracks1_len = len(self._tracks1)
+        tracks2_len = len(self._tracks2)
+        if src in self._tracks1:
+            src_widget = self._tracks_widget_left
+            src_index = self._tracks1.index(src) - 1
+        else:
+            src_widget = self._tracks_widget_right
+            src_index = self._tracks2.index(src) - 1
+        if tracks1_len == 0 or dst in self._tracks1:
+            dst_widget = self._tracks_widget_left
+            dst_tracks = self._tracks1
+        elif tracks2_len == 0 or dst in self._tracks2:
+            dst_widget = self._tracks_widget_right
+            dst_tracks = self._tracks2
+        else:
             return
+        # Remove src from src_widget
+        for child in src_widget.get_children():
+            if child.get_id() == src:
+                child.destroy()
+                break
+        src_track = Track(src)
+        prev_track = Track()
+        name = escape(src_track.name)
+        index = 0
+        # Get previous track
+        if dst != -1:
+            for child in dst_widget.get_children():
+                if child.get_id() == dst:
+                    break
+                index += 1
+            if not up:
+                index += 1
+            # Get previous track (in dst context)
+            prev_index = dst_tracks.index(dst)
+            if up:
+                prev_index -= 1
+            prev_track = Track(dst_tracks[prev_index])
+            # If we are listening to a compilation, prepend artist name
+            if (src_track.album.artist_id == Type.COMPILATIONS or
+                    len(src_track.artist_ids) > 1 or
+                    src_track.album.artist_id not in src_track.artist_ids):
+                name = "<b>%s</b>\n%s" % (escape(src_track.artist_names), name)
+            self._tracks1.insert(index, src_track.id)
+        row = PlaylistRow(src_track.id,
+                          index,
+                          index == 0 or
+                          src_track.album.id != prev_track.album.id)
+        row.connect('track-moved', self._on_track_moved)
+        row.show()
+        dst_widget.insert(row, index)
+        return (src_widget, dst_widget, src_index, index)
 
-        # If we are listening to a compilation, prepend artist name
-        if (album.artist_id == Type.COMPILATIONS or
-                len(track.artist_ids) > 1 or
-                album.artist_id not in track.artist_ids):
-            name = "<b>%s</b>\n%s" % (escape(track.artist_names), name)
+    def _on_track_moved(self, widget, dst, src, up):
+        """
+            Move track from src to row
+            Recalculate track position
+            @param widget as TracksWidget
+            @param dst as int
+            @param src as int
+            @param up as bool
+        """
+        (src_widget, dst_widget, src_index, dst_index) = \
+            self._move_track(dst, src, up)
+        self._update_tracks()
+        self._update_position()
+        self._update_headers()
+        self._tracks_widget_left.update_indexes(1)
+        self._tracks_widget_right.update_indexes(len(self._tracks1) + 1)
+        # Save playlist in db only if one playlist visible
+        if len(self._playlist_ids) == 1 and self._playlist_ids[0] >= 0:
+            Lp().playlists.clear(self._playlist_ids[0], False)
+            tracks = []
+            for track_id in self._tracks1 + self._tracks2:
+                tracks.append(Track(track_id))
+            Lp().playlists.add_tracks(self._playlist_ids[0],
+                                      tracks,
+                                      False)
+        if Lp().player.get_user_playlist_ids() == self._playlist_ids:
+            Lp().player.populate_user_playlist_by_tracks(self._tracks1 +
+                                                         self._tracks2,
+                                                         self._playlist_ids)
 
-        if widget == self._tracks_widget1:
-            self._tracks1.append(track)
+    def _on_size_allocate(self, widget, allocation):
+        """
+            Change box max/min children
+            @param widget as Gtk.Widget
+            @param allocation as Gtk.Allocation
+        """
+        if self._width == allocation.width:
+            return
+        self._width = allocation.width
+        redraw = False
+        if allocation.width < WindowSize.MONSTER:
+            orientation = Gtk.Orientation.VERTICAL
         else:
-            self._tracks2.append(track)
-
-        if album.id != previous_album_id:
-            widget.add_album(track.id, album, pos,
-                             name, track.duration, None)
-        else:
-            widget.add_album(track.id, None, pos, name, track.duration, None)
-        GLib.idle_add(self._add_tracks, tracks, widget, pos + 1, album.id)
+            orientation = Gtk.Orientation.HORIZONTAL
+        if orientation != self._orientation:
+            self._orientation = orientation
+            redraw = True
+        self._box.set_orientation(orientation)
+        if redraw:
+            for child in self._box.get_children():
+                self._box.remove(child)
+            GLib.idle_add(self._box.add, self._tracks_widget_left)
+            GLib.idle_add(self._box.add, self._tracks_widget_right)
+        self._update_headers()
 
     def _on_activated(self, widget, track_id):
         """
@@ -179,16 +380,11 @@ class PlaylistWidget(Gtk.Bin):
             @param widget as TracksWidget
             @param track as Track
         """
-        if Lp().player.is_party():
-            Lp().player.load(Track(track_id))
-        elif self._playlist_id < 0:
-            Lp().player.load(Track(track_id))
-            Lp().player.set_user_playlist_id(self._playlist_id)
-            Lp().player.set_user_playlist_by_tracks(self._tracks1 +
-                                                    self._tracks2)
-        else:
-            Lp().player.set_user_playlist_by_id(self._playlist_id)
-            Lp().player.load_in_playlist(track_id)
+        Lp().player.load(Track(track_id))
+        if not Lp().player.is_party():
+            Lp().player.populate_user_playlist_by_tracks(self._tracks1 +
+                                                         self._tracks2,
+                                                         self._playlist_ids)
 
 
 class PlaylistsManagerWidget(Gtk.Bin):
@@ -464,7 +660,6 @@ class PlaylistEditWidget(Gtk.Bin):
         """
         Gtk.Bin.__init__(self)
         self._playlist_id = playlist_id
-        self._save_on_disk = True
 
         builder = Gtk.Builder()
         builder.add_from_resource('/org/gnome/Lollypop/PlaylistEditWidget.ui')
@@ -479,7 +674,6 @@ class PlaylistEditWidget(Gtk.Bin):
                                     str,
                                     str,
                                     int)
-        self._model.connect("row-deleted", self._on_row_deleted)
 
         self._view.set_model(self._model)
 
@@ -551,41 +745,6 @@ class PlaylistEditWidget(Gtk.Bin):
             self._view.grab_focus()
             self._in_thread = False
 
-    def _update_on_disk(self):
-        """
-            Update playlist on disk
-        """
-        def update():
-            Lp().playlists.clear(self._playlist_id, False)
-            tracks = []
-            for item in self._model:
-                tracks.append(Track(item[3]))
-            Lp().playlists.add_tracks(self._playlist_id, tracks)
-        t = Thread(target=update)
-        t.daemon = True
-        t.start()
-
-    def _show_infobar(self, path):
-        """
-            Show infobar
-            @param path as Gtk.TreePath
-        """
-        iterator = self._model.get_iter(path)
-        self._infobar_label.set_markup(_("Remove \"%s\"?") %
-                                       self._model.get_value(iterator,
-                                                             1).replace('\n',
-                                                                        ' - '))
-        self._infobar.show()
-
-    def _on_keyboard_event(self, widget, event):
-        """
-            Delete item if Delete was pressed
-            @param widget unused, Gdk.Event
-        """
-        if event.keyval == 65535:
-            path, column = self._view.get_cursor()
-            self._show_infobar(path)
-
     def _on_response(self, infobar, response_id):
         """
             Hide infobar
@@ -615,45 +774,46 @@ class PlaylistEditWidget(Gtk.Bin):
             @param selection as Gtk.TreeSelection
         """
         count = selection.count_selected_rows()
-        if count > 1:
+        if count == 1:
+            (model, path) = selection.get_selected_rows()
+            iterator = model.get_iter(path)
+            self._infobar_label.set_markup(_("Remove \"%s\"?") %
+                                           model.get_value(iterator,
+                                                           1).replace('\n',
+                                                                      ' - '))
+            self._infobar.show()
+        elif count > 0:
             self._infobar_label.set_markup(_("Remove these tracks?"))
             self._infobar.show()
-
-    def _on_row_deleted(self, path, data):
-        """
-            Only catch drag & drop successful
-            @param path as Gtk.TreePath
-            @param data as unused
-        """
-        if self._save_on_disk:
-            self._update_on_disk()
+        else:
+            self._infobar.hide()
 
     def _on_delete_confirm(self, button):
         """
             Delete tracks after confirmation
             @param button as Gtk.Button
         """
-        self._save_on_disk = False
         selection = self._view.get_selection()
-        selected = selection.get_selected_rows()
+        selected = selection.get_selected_rows()[1]
         rows = []
-        for item in selected[1]:
+        for item in selected:
             rows.append(Gtk.TreeRowReference.new(self._model, item))
 
+        tracks = []
         for row in rows:
             iterator = self._model.get_iter(row.get_path())
             track = Track(self._model.get_value(iterator, 3))
-            if track.album.artist_id == Type.COMPILATIONS:
-                artist_name = track.artist_names
-            else:
-                artist_name = track.album.artist_name
-            if Lp().lastfm is not None:
+            tracks.append(track)
+            if self._playlist_id == Type.LOVED and Lp().lastfm is not None:
+                if track.album.artist_id == Type.COMPILATIONS:
+                    artist_name = track.artist_names
+                else:
+                    artist_name = track.album.artist_name
                 t = Thread(target=Lp().lastfm.unlove,
                            args=(artist_name, track.name))
                 t.daemon = True
                 t.start()
             self._model.remove(iterator)
+        Lp().playlists.remove_tracks(self._playlist_id, tracks)
         self._infobar.hide()
-        self._save_on_disk = True
         self._unselectall()
-        self._update_on_disk()
