@@ -37,7 +37,7 @@ class Window(Gtk.ApplicationWindow, Container):
         Gtk.ApplicationWindow.__init__(self,
                                        application=app,
                                        title="Lollypop")
-        self._nullwidget = Gtk.Label()  # Use to get selected background color
+        self.connect('notify::is-active', self._on_active)
         self._timeout_configure = None
         seek_action = Gio.SimpleAction.new('seek',
                                            GLib.VariantType.new('i'))
@@ -67,14 +67,7 @@ class Window(Gtk.ApplicationWindow, Container):
             Add an application menu to window
             @parma: menu as Gio.Menu
         """
-        self._toolbar.setup_menu_btn(menu)
-
-    def get_selected_color(self):
-        """
-            Return selected color
-        """
-        return self._nullwidget.get_style_context().\
-            get_background_color(Gtk.StateFlags.SELECTED)
+        self._toolbar.setup_menu(menu)
 
     def enable_global_shorcuts(self, enable):
         """
@@ -165,8 +158,6 @@ class Window(Gtk.ApplicationWindow, Container):
         size = self.get_size()
         self._toolbar.set_content_width(size[0])
         view = self._stack.get_visible_child()
-        if view:
-            view.update_responsive_design()
         if view and hasattr(view, 'show_context'):
             view.show_context(size[0] > WindowSize.MEDIUM)
         if Lp().player.current_track.id is not None:
@@ -232,7 +223,8 @@ class Window(Gtk.ApplicationWindow, Container):
             self._toolbar.set_show_close_button(False)
         elif mini is not None and not show and self._timeout is None:
             self._main_stack.set_visible_child_name('main')
-            self._toolbar.set_show_close_button(True)
+            self._toolbar.set_show_close_button(
+                                    not Lp().settings.get_value('disable-csd'))
             self._timeout = GLib.timeout_add(1000, mini.destroy)
 
     def _setup_pos_size(self, name):
@@ -320,7 +312,7 @@ class Window(Gtk.ApplicationWindow, Container):
         """
             Setup window content
         """
-        self.set_icon_name('lollypop')
+        self.set_default_icon_name('lollypop')
         vgrid = Gtk.Grid()
         vgrid.set_orientation(Gtk.Orientation.VERTICAL)
         vgrid.show()
@@ -435,7 +427,7 @@ class Window(Gtk.ApplicationWindow, Container):
                     Lp().settings.get_enum('shuffle') == Shuffle.TRACKS:
                 Lp().player.set_next()
                 # We send this signal to update next popover
-                Lp().player.emit("queue-changed")
+                Lp().player.emit('queue-changed')
             else:
                 Lp().player.context.next = NextContext.START_NEW_ALBUM
                 Lp().player.set_next()
@@ -453,3 +445,12 @@ class Window(Gtk.ApplicationWindow, Container):
             # No idea why, maybe scanner using Gstpbutils before Gstreamer
             # initialisation is finished...
             GLib.timeout_add(2000, self.update_db)
+
+    def _on_active(self, window, active):
+        """
+            Clean overlays if not active
+            @param widget as Gtk.Window
+            @param active as boolean
+        """
+        if not window.is_active():
+            self.update_overlays()
